@@ -773,7 +773,9 @@ void scale32_core(PyObject *pysrc, PyObject *pydst,
                   float source_xoff, float source_yoff,
                   float source_width, float source_height,
                   float dest_xoff, float dest_yoff,
-                  float dest_width, float dest_height) {
+                  float dest_width, float dest_height,
+                  int precise
+    ) {
 
 
     SDL_Surface *src;
@@ -803,9 +805,25 @@ void scale32_core(PyObject *pysrc, PyObject *pydst,
     srch = src->h;
     dsth = dst->h;
 
-    xdelta = 255.0 * (source_width - 1) / dest_width;
-    ydelta = 255.0 * (source_height - 1) / dest_height;
+    if (precise) {
 
+        if (dest_width > 1) {
+            xdelta = 256.0 * (source_width - 1) / (dest_width - 1);
+        } else {
+            xdelta = 0;
+        }
+
+        if (dest_height > 1) {
+            ydelta = 256.0 * (source_height - 1) / (dest_height - 1);
+        } else {
+            ydelta = 0;
+        }
+        
+    } else {
+        xdelta = 255.0 * (source_width - 1) / dest_width;
+        ydelta = 255.0 * (source_height - 1) / dest_height;
+    }
+        
     for (y = 0; y < dsth; y++) {
 
         unsigned char *s0;
@@ -821,14 +839,14 @@ void scale32_core(PyObject *pysrc, PyObject *pydst,
         d = dstpixels + dstpitch * y;
         dend = d + 4 * dstw; // bpp
 
-        sline = source_yoff * 255 + (y + dest_yoff) * ydelta;
+        sline = source_yoff * 256 + (y + dest_yoff) * ydelta;
         s1frac = (int) sline & 255;
         s0frac = 256 - s1frac;
 
         s0 = srcpixels + (sline >> 8) * srcpitch;
         s1 = s0 + srcpitch;
 
-        scol = source_xoff * 255 + dest_xoff * xdelta;
+        scol = source_xoff * 256 + dest_xoff * xdelta;
 
         while (d < dend) {
 
@@ -960,15 +978,22 @@ void scale24_core(PyObject *pysrc, PyObject *pydst,
 }
 
 #define I(a, b, mul) ((((((b - a) * mul)) >> 8) + a) & 0xff00ff)
-                
+
+/** This appears to limit the expansion, such that 1/x yields a max
+    expansion of lg x */
+#define EPSILON (1.0 / 256.0)
+
 /****************************************************************************/
 /* A similar concept to rotozoom, but implemented differently, so we
    can limit the target area. */
-void transform32_std(PyObject *pysrc, PyObject *pydst,
-                     float corner_x, float corner_y,
-                     float xdx, float ydx,
-                     float xdy, float ydy,
-                     int ashift, float a) {
+int transform32_std(PyObject *pysrc, PyObject *pydst,
+                    float corner_x, float corner_y,
+                    float xdx, float ydx,
+                    float xdy, float ydy,
+                    int ashift,
+                    float a,
+                    int precise
+    ) {
 
     SDL_Surface *src;
     SDL_Surface *dst;
@@ -978,12 +1003,13 @@ void transform32_std(PyObject *pysrc, PyObject *pydst,
     int srcw, srch;
     int dstw, dsth;
     
+    // The x and y source pixel coordinates, times 65536. And their
+    // delta-per-dest-x-pixel.
+    int sxi = 0, syi = 0, dsxi = 0, dsyi = 0;
+
     unsigned char *srcpixels;
     unsigned char *dstpixels;
 
-    float lsx, lsy; // The position of the current line in the source.
-    float sx, sy; // The position of the current pixel in the source.
-    
     src = PySurface_AsSurface(pysrc);
     dst = PySurface_AsSurface(pydst);
         
@@ -998,81 +1024,113 @@ void transform32_std(PyObject *pysrc, PyObject *pydst,
     srch = src->h;
     dsth = dst->h;
 
-    unsigned int amul = (int) (a * 256);
+    // Compute the coloring multiplier.
+    unsigned int amul = (unsigned int) (a * 256);
+
+    // Compute the maximum x and y coordinates.
+    double maxsx = srcw - 1;
+    double maxsy = srch - 1;
     
-    lsx = corner_x * 65536;
-    lsy = corner_y * 65536;
+    // Deal with pre-6.10.1 versions of Ren'Py, which didn't give us
+    // that 1px border that allows us to be precise.
+    if (! precise) {
+        maxsx -= EPSILON;
+        maxsy -= EPSILON;
+    
+        // If a delta is too even, subtract epsilon (towards 0) from it.
+        if (xdx && fabs(fmodf(1.0 / xdx, 1)) < EPSILON) {
+            xdx -= (xdx / fabs(xdx)) * EPSILON;
+        }
+        if (xdy && fabs(fmodf(1.0 / xdy, 1)) < EPSILON) {
+            xdy -= (xdy / fabs(xdy)) * EPSILON;
+        }
+        if (ydx && fabs(fmodf(1.0 / ydx, 1)) < EPSILON) {
+            ydx -= (ydx / fabs(ydx)) * EPSILON;
+        }
+        if (ydy && fabs(fmodf(1.0 / ydy, 1)) < EPSILON) {
+            ydy -= (ydy / fabs(ydy)) * EPSILON;
+        }
+    }
 
-    xdx *= 65536;
-    ydx *= 65536;
-    xdy *= 65536;
-    ydy *= 65536;
-
-
-    // Scaled subtracted srcw and srch.
-    float fsw = (srcw - 1) * 65536 - 1;
-    float fsh = (srch - 1) * 65536 - 1;
-
-    for (y = 0; y < dsth; y++, lsx += xdy, lsy += ydy) {
-
-        sx = lsx;
-        sy = lsy;
+    
+    // Loop through every line.
+    for (y = 0; y < dsth; y++) {
         
-        // unsigned char *d = dstpixels + dstpitch * y;
-        // unsigned char *dend = d + 4 * dstw;
+        // The source coordinates of the leftmost pixel in the line.
+        double leftsx = corner_x + y * xdy;
+        double leftsy = corner_y + y * ydy;
 
-        float minx = 0;
-        float maxx = dstw - 1;
+        // Min and max x-extent to draw on the current line.
+        double minx = 0;
+        double maxx = dstw - 1;
 
-        if (xdx != 0) {
-            float d1 = -lsx / xdx;
-            float d2 = (fsw - lsx) / xdx;
+        // Figure out the x-extent based on xdx.
+        if (xdx) {
+            double x1 = (0.0 - leftsx) / xdx;
+            double x2 = (maxsx - leftsx) / xdx;
 
-            minx = fmaxf(minx, fminf(d1, d2)); 
-            maxx = fminf(maxx, fmaxf(d1, d2)); 
+            if (x1 < x2) {
+                minx = fmax(x1, minx);
+                maxx = fmin(x2, maxx);                
+            } else {
+                minx = fmax(x2, minx);
+                maxx = fmin(x1, maxx);                
+            }
+
+        } else {
+            if (leftsx < 0 || leftsx > maxsx) {
+               continue;
+            }
+        }
+
+        // Figure out the x-extent based on ydx.
+        if (ydx) {
+            double x1 = (0.0 - leftsy) / ydx;
+            double x2 = (maxsy - leftsy) / ydx;
+
+            if (x1 < x2) {
+                minx = fmax(x1, minx);
+                maxx = fmin(x2, maxx);                
+            } else {
+                minx = fmax(x2, minx);
+                maxx = fmin(x1, maxx);                
+            }
             
-        } else if ( lsx < 0 || lsx >= fsw) {
-            continue;
+        } else {
+            if (leftsy < 0 || leftsy > maxsy) {
+                continue;
+            }
         }
-
-        if (ydx != 0) {
-            float d1 = -lsy / ydx;
-            float d2 = (fsh - lsy) / ydx;
-
-            minx = fmaxf(minx, fminf(d1, d2)); 
-            maxx = fminf(maxx, fmaxf(d1, d2)); 
-        } else if ( lsy < 0 || lsy >= fsh) {
-            continue;
-        }
-
-        if (minx > maxx) {
-            continue;
-        }
-
+                
         minx = ceil(minx);
         maxx = floor(maxx);
 
+        if (minx >= maxx) {
+            continue;
+        }
+        
+        // The start and end of line pointers.
         unsigned char *d = dstpixels + dstpitch * y;
         unsigned char *dend = d + 4 * (int) maxx;
+
+        // Advance start of line by 4.
         d += 4 * (int) minx;
 
-        sx = lsx + minx * xdx;
-        sy = lsy + minx * ydx;
-
-        int sxi = (int) sx;
-        int syi = (int) sy;
-        int xdxi = (int) xdx;
-        int ydxi = (int) ydx;
+        // Starting coordinates and deltas.
+        sxi = (int) ((leftsx + minx * xdx) * 65536); 
+        syi = (int) ((leftsy + minx * ydx) * 65536);
+        dsxi = (int) (xdx * 65536);
+        dsyi = (int) (ydx * 65536);
 
         while (d <= dend) {
-            int px, py;
-            px = sxi >> 16;
-            py = syi >> 16;
-                
+
+            int px = sxi >> 16;
+            int py = syi >> 16;
+
             unsigned char *sp = srcpixels + py * srcpitch + px * 4;
 
-            int yfrac = (syi >> 8) & 0xff; // ((short) sy) & 0xff;
-            int xfrac = (sxi >> 8) & 0xff; // ((short) sx) & 0xff;
+            unsigned int yfrac = (syi >> 8) & 0xff; // ((short) sy) & 0xff;
+            unsigned int xfrac = (sxi >> 8) & 0xff; // ((short) sx) & 0xff;
 
             unsigned int pal = *(unsigned int *) sp;
             unsigned int pbl = *(unsigned int *) (sp + 4);
@@ -1104,46 +1162,53 @@ void transform32_std(PyObject *pysrc, PyObject *pydst,
             dh = I(dh, rh, alpha);
             
             * (unsigned int *) d = (dh << 8) | dl;
-
+            
             d += 4;
-            sxi += xdxi;
-            syi += ydxi;
+            sxi += dsxi;
+            syi += dsyi;
         }
+
     }
 
-    Py_END_ALLOW_THREADS
+    Py_END_ALLOW_THREADS;
+
+
+    // This is bogus, and only serves to ensure that the FPU
+    // computes these variables at the right times.
+    return sxi + syi + dsxi + dsyi;
 }
 
+
+
 #ifdef GCC_MMX
 
 /****************************************************************************/
 /* A similar concept to rotozoom, but implemented differently, so we
    can limit the target area. */
 int transform32_mmx(PyObject *pysrc, PyObject *pydst,
-                     float corner_x, float corner_y,
-                     float xdx, float ydx,
-                     float xdy, float ydy,
-                     int ashift,
-                     float a) {
+                    float corner_x, float corner_y,
+                    float xdx, float ydx,
+                    float xdy, float ydy,
+                    int ashift,
+                    float a,
+                    int precise
+    ) {
 
     SDL_Surface *src;
     SDL_Surface *dst;
     
     int y;
     int srcpitch, dstpitch;
-                               
     int srcw, srch;
     int dstw, dsth;
     
+    // The x and y source pixel coordinates, times 65536. And their
+    // delta-per-dest-x-pixel.
+    int sxi = 0, syi = 0, dsxi = 0, dsyi = 0;
+
     unsigned char *srcpixels;
     unsigned char *dstpixels;
 
-    float lsx, lsy; // The position of the current line in the source.
-    float sx, sy; // The position of the current pixel in the source.
-    unsigned int px, py;
-    unsigned int sxi, syi;
-    unsigned int xdxi, ydxi;
-        
     src = PySurface_AsSurface(pysrc);
     dst = PySurface_AsSurface(pydst);
         
@@ -1163,63 +1228,101 @@ int transform32_mmx(PyObject *pysrc, PyObject *pydst,
 
     // Compute the coloring multiplier.
     unsigned int amul = (unsigned int) (a * 256);
+
+    // Compute the maximum x and y coordinates.
+    double maxsx = srcw - 1;
+    double maxsy = srch - 1;
     
-    lsx = corner_x * 65536;
-    lsy = corner_y * 65536;
-
-    xdx *= 65536;
-    ydx *= 65536;
-    xdy *= 65536;
-    ydy *= 65536;
-
-    // Scaled subtracted srcw and srch.
-    float fsw = (srcw - 1) * 65536 - 1;
-    float fsh = (srch - 1) * 65536 - 1;
-
-    for (y = 0; y < dsth; y++, lsx += xdy, lsy += ydy) {
-
-        float minx = 0;
-        float maxx = dstw - 1;
-
-        if (xdx != 0) {
-            float d1 = -lsx / xdx;
-            float d2 = (fsw - lsx) / xdx;
-
-            minx = fmaxf(minx, fminf(d1, d2)); 
-            maxx = fminf(maxx, fmaxf(d1, d2)); 
-            
-        } else if ( lsx < 0 || lsx >= fsw) {
-            continue;
+    // Deal with pre-6.10.1 versions of Ren'Py, which didn't give us
+    // that 1px border that allows us to be precise.
+    if (! precise) {
+        maxsx -= EPSILON;
+        maxsy -= EPSILON;
+    
+        // If a delta is too even, subtract epsilon (towards 0) from it.
+        if (xdx && fabs(fmodf(1.0 / xdx, 1)) < EPSILON) {
+            xdx -= (xdx / fabs(xdx)) * EPSILON;
         }
-
-        if (ydx != 0) {
-            float d1 = -lsy / ydx;
-            float d2 = (fsh - lsy) / ydx;
-
-            minx = fmaxf(minx, fminf(d1, d2)); 
-            maxx = fminf(maxx, fmaxf(d1, d2)); 
-        } else if ( lsy < 0 || lsy >= fsh) {
-            continue;
+        if (xdy && fabs(fmodf(1.0 / xdy, 1)) < EPSILON) {
+            xdy -= (xdy / fabs(xdy)) * EPSILON;
         }
-
-        if (minx > maxx) {
-            continue;
+        if (ydx && fabs(fmodf(1.0 / ydx, 1)) < EPSILON) {
+            ydx -= (ydx / fabs(ydx)) * EPSILON;
         }
+        if (ydy && fabs(fmodf(1.0 / ydy, 1)) < EPSILON) {
+            ydy -= (ydy / fabs(ydy)) * EPSILON;
+        }
+    }
+
+    
+    // Loop through every line.
+    for (y = 0; y < dsth; y++) {
         
+        // The source coordinates of the leftmost pixel in the line.
+        double leftsx = corner_x + y * xdy;
+        double leftsy = corner_y + y * ydy;
+
+        // Min and max x-extent to draw on the current line.
+        double minx = 0;
+        double maxx = dstw - 1;
+
+        // Figure out the x-extent based on xdx.
+        if (xdx) {
+            double x1 = (0.0 - leftsx) / xdx;
+            double x2 = (maxsx - leftsx) / xdx;
+
+            if (x1 < x2) {
+                minx = fmax(x1, minx);
+                maxx = fmin(x2, maxx);                
+            } else {
+                minx = fmax(x2, minx);
+                maxx = fmin(x1, maxx);                
+            }
+
+        } else {
+            if (leftsx < 0 || leftsx > maxsx) {
+               continue;
+            }
+        }
+
+        // Figure out the x-extent based on ydx.
+        if (ydx) {
+            double x1 = (0.0 - leftsy) / ydx;
+            double x2 = (maxsy - leftsy) / ydx;
+
+            if (x1 < x2) {
+                minx = fmax(x1, minx);
+                maxx = fmin(x2, maxx);                
+            } else {
+                minx = fmax(x2, minx);
+                maxx = fmin(x1, maxx);                
+            }
+            
+        } else {
+            if (leftsy < 0 || leftsy > maxsy) {
+                continue;
+            }
+        }
+
         minx = ceil(minx);
         maxx = floor(maxx);
+
+        if (minx >= maxx) {
+            continue;
+        }
         
+        // The start and end of line pointers.
         unsigned char *d = dstpixels + dstpitch * y;
         unsigned char *dend = d + 4 * (int) maxx;
+
+        // Advance start of line by 4.
         d += 4 * (int) minx;
 
-        sx = (lsx + minx * xdx);
-        sy = (lsy + minx * ydx);
-            
-        sxi = (unsigned int) sx;
-        syi = (unsigned int) sy;
-        xdxi = (int) xdx;
-        ydxi = (int) ydx;
+        // Starting coordinates and deltas.
+        sxi = (int) ((leftsx + minx * xdx) * 65536); 
+        syi = (int) ((leftsy + minx * ydx) * 65536);
+        dsxi = (int) (xdx * 65536);
+        dsyi = (int) (ydx * 65536);
 
         // No floating point allowed between here and the end of the
         // while loop.
@@ -1235,8 +1338,8 @@ int transform32_mmx(PyObject *pysrc, PyObject *pydst,
         
         while (d <= dend) {
 
-            px = sxi >> 16;
-            py = syi >> 16;
+            int px = sxi >> 16;
+            int py = syi >> 16;
 
             unsigned char *sp = srcpixels + py * srcpitch + px * 4;
 
@@ -1310,19 +1413,19 @@ int transform32_mmx(PyObject *pysrc, PyObject *pydst,
             movd_r2m(mm1, *(unsigned int *)d);
             
             d += 4;
-            sxi += xdxi;
-            syi += ydxi;
+            sxi += dsxi;
+            syi += dsyi;
         }
 
         emms();
     }
 
-    Py_END_ALLOW_THREADS
+    Py_END_ALLOW_THREADS;
 
 
-        // This is bogus, and only serves to ensure that the FPU
-        // computes these variables at the right times.
-        return sxi + syi + xdxi + ydxi;
+    // This is bogus, and only serves to ensure that the FPU
+    // computes these variables at the right times.
+    return sxi + syi + dsxi + dsyi;
 }
 
 #endif
@@ -1332,7 +1435,9 @@ void transform32_core(PyObject *pysrc, PyObject *pydst,
                       float xdx, float ydx,
                       float xdy, float ydy,
                       int ashift,
-                      float a) {
+                      float a,
+                      int precise
+    ) {
     
 #ifdef GCC_MMX
     static int checked_mmx = 0;
@@ -1345,14 +1450,14 @@ void transform32_core(PyObject *pysrc, PyObject *pydst,
 
     if (has_mmx) {
         transform32_mmx(pysrc, pydst, corner_x, corner_y,
-                        xdx, ydx, xdy, ydy, ashift, a);
+                        xdx, ydx, xdy, ydy, ashift, a, precise);
         return;
     }
     
 #endif
     
     transform32_std(pysrc, pydst, corner_x, corner_y,
-                    xdx, ydx, xdy, ydy, ashift, a);
+                    xdx, ydx, xdy, ydy, ashift, a, precise);
 
 }
                      
