@@ -2,12 +2,6 @@ import sys
 
 from sound import *
 
-try:
-    import nativemidi
-    sys.modules['nativemidi'] = sys.modules['pysdlsound.nativemidi']
-except:
-    pass
-
 try:
     import winmixer
     sys.modules['winmixer'] = sys.modules['pysdlsound.winmixer']
