@@ -1,5 +1,5 @@
 /*
-Copyright 2005 PyTom <pytom@bishoujo.us>
+Copyright 2005-2009 PyTom <pytom@bishoujo.us>
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation files
@@ -24,9 +24,19 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "pss.h"
 #include <Python.h>
 #include <SDL/SDL.h>
-#include <SDL/SDL_sound.h>
 #include <stdio.h>
 
+/* Declarations of ffdecode functions. */
+struct VideoState;
+
+struct VideoState *ffpy_stream_open(SDL_RWops *, const char *);
+void ffpy_stream_close(struct VideoState *is);
+void ffpy_alloc_event(struct VideoState *vs, PyObject *surface);
+void ffpy_refresh_event(struct VideoState *vs);
+void ffpy_init(int rate, int status);
+int ffpy_audio_decode(struct VideoState *is, Uint8 *stream, int len);
+
+    
 /* The current Python. */
 PyInterpreterState* interp;
 PyThreadState* thread = NULL;
@@ -52,6 +62,10 @@ static void decref(PyObject *ref) {
 }
 
 /* Locking on entry from python... */
+// #define BEGIN() PyThreadState *_save;
+// #define ENTER() { printf("Locking by %s.\n", __FUNCTION__); _save = PyEval_SaveThread(); SDL_LockAudio(); printf("Lock by %s\n", __FUNCTION__);  }
+// #define EXIT() { SDL_UnlockAudio(); PyEval_RestoreThread(_save); printf("Release by %s\n", __FUNCTION__); }
+
 #define BEGIN() PyThreadState *_save;
 #define ENTER() { _save = PyEval_SaveThread(); SDL_LockAudio(); }
 #define EXIT() { SDL_UnlockAudio(); PyEval_RestoreThread(_save); }
@@ -60,9 +74,6 @@ static void decref(PyObject *ref) {
 #define min(a, b) (((a) < (b)) ? (a) : (b))
 #define max(a, b) (((a) > (b)) ? (a) : (b))
 
-/* The number of channels we support. */
-#define NUM_CHANNELS 8
-
 /* Various error codes. */
 #define SUCCESS 0
 #define SDL_ERROR -1
@@ -86,7 +97,7 @@ struct Channel {
 
     /* The currently playing sample, NULL if this sample isn't playing
        anything. */
-    Sound_Sample *playing;
+    struct VideoState *playing;
 
     /* The name of the playing music. */
     PyObject *playing_name;
@@ -98,7 +109,7 @@ struct Channel {
     int playing_tight;
     
     /* The queued up sample. */
-    Sound_Sample *queued;    
+    struct VideoState *queued;    
 
     /* The name of the queued up sample. */
     PyObject *queued_name;
@@ -115,12 +126,6 @@ struct Channel {
     /* The volume of the channel. */
     int volume;
 
-    /* The number of decoded bytes in the buffer. */
-    int decoded;
-
-    /* The offset into the buffer that we last stopped at. */
-    int last;
-
     /* The position (in bytes) that this channel has queued to. */
     int pos;
 
@@ -145,12 +150,41 @@ struct Channel {
     /* The event posted to the queue when we finish a track. */
     int event;
 
+    /* The pan being applied to the current channel. */
+    float pan_start;
+    float pan_end;
+
+    /* The length of the current pan, in samples. */
+    unsigned int pan_length;
+
+    /* The number of samples we've finished in the current pan. */
+    unsigned int pan_done;
+
+
+    /* These are used like in pan, above. Unlike the volume parameter,
+       the voulme set here is persisted between sessions. */
+    float vol2_start;
+    float vol2_end;
+    unsigned int vol2_length;
+    unsigned int vol2_done;
 };
 
+struct Dying {
+    struct VideoState *stream;
+    struct Dying *next;
+};
+
+static struct Dying *dying = NULL;
+
+/*
+ * The number of channels the system knows about.
+ */
+int num_channels = 0;
+
 /*
  * All of the channels that the system knows about.
  */
-struct Channel channels[NUM_CHANNELS];
+struct Channel *channels = NULL;
 
 /*
  * The spec of the audio that is playing.
@@ -158,6 +192,39 @@ struct Channel channels[NUM_CHANNELS];
 SDL_AudioSpec audio_spec;
 
 
+static float interpolate_pan(struct Channel *c) {
+    float done;
+
+    if (c->pan_done > c->pan_length) {
+        c->pan_length = 0;
+    }
+
+    if (c->pan_length == 0) {
+        return c->pan_end;
+    }
+
+    done = 1.0 * c->pan_done / c->pan_length;
+
+    return c->pan_start + done * (c->pan_end - c->pan_start);
+    
+}
+
+static float interpolate_vol2(struct Channel *c) {
+    float done;
+
+    if (c->vol2_done > c->vol2_length) {
+        c->vol2_length = 0;
+    }
+
+    if (c->vol2_length == 0) {
+        return c->vol2_end;
+    }
+
+    done = 1.0 * c->vol2_done / c->vol2_length;
+
+    return c->vol2_start + done * (c->vol2_end - c->vol2_start);
+}
+
 static int ms_to_bytes(int ms) {
     return ((long long) ms) * audio_spec.freq * audio_spec.channels * 2 / 1000;
 }
@@ -171,8 +238,6 @@ static void start_sample(struct Channel* c, int reset_fade) {
 
     if (!c) return;
 
-    c->decoded = 0;
-    c->last = 0;
     c->pos = 0;
 
     if (reset_fade) {
@@ -185,62 +250,52 @@ static void start_sample(struct Channel* c, int reset_fade) {
             c->fade_off = 0;
             c->fade_vol = 0;
 
-            c->fade_step_len = ms_to_bytes(c->playing_fadein) / fade_steps;
-            c->fade_step_len &= ~0x7; // Even sample.
-
+            if (fade_steps) {
+                c->fade_step_len = ms_to_bytes(c->playing_fadein) / fade_steps;
+                c->fade_step_len &= ~0x7; // Even sample.
+            } else {
+                c->fade_step_len = 0;
+            }
         }
 
         c->stop_bytes = -1;
     } 
 }
 
-
-/* static void update_pause(void) { */
-/*     int i; */
-/*     int pause = 1; */
-    
-/*     for (i = 0; i < NUM_CHANNELS; i++) { */
-/*         if (channels[i].playing) { */
-/*             pause = 0; */
-/*             break; */
-/*         } */
-/*     } */
-
-/*     SDL_PauseAudio(pause); */
-/* } */
-
-static void free_sample(Sound_Sample *ss) {
-    Sound_FreeSample(ss);
+static void free_sample(struct VideoState *ss) {
+    ffpy_stream_close(ss);
 }
 
+#define MAX_SHORT (32767)
+#define MIN_SHORT (-32768)
+
 // Actually mixes the audio.
 static void mixaudio(Uint8 *dst, Uint8 *src, int length, int volume) {
- 
-    // SDL_MixAudio may not work when length % 16 != 0.
-    if ((length & 0x0f) == 0) {
-        SDL_MixAudio(dst, src, length, volume);
-    } else {
+    int i;
+    short *sdst = (short *) dst;
+    short *ssrc = (short *) src;
 
-        int newlength = length + 16 - (length & 0xf);
-        Uint8 newsrc[newlength];
-        Uint8 newdst[newlength];
-        
-        memcpy(newsrc, src, length);
-        memcpy(newdst, dst, length);
+    for (i = 0; i < length / 2; i++) {
+        int sound = *sdst + (volume * *ssrc) / SDL_MIX_MAXVOLUME;
+        if (sound > MAX_SHORT) {
+            sound = MAX_SHORT;
+        }
+        if (sound < MIN_SHORT) {
+            sound = MIN_SHORT;
+        }
 
-        // Mix the audio once.
-        SDL_MixAudio(newdst, newsrc, newlength, volume);
-
-        memcpy(dst, newdst, length);
-    }    
+        *sdst++ = (short) sound;
+        ssrc++;        
+    }
 }
 
+
 // Mixes the audio, while performing fading.
 static void fade_mixaudio(struct Channel *c,
                           Uint8 *dst, Uint8 *src, int length) {
 
     while (length) {
-
+        
         // No fade case.
         if (c->fade_step_len == 0) {
             mixaudio(dst, src, length, c->volume);
@@ -250,8 +305,9 @@ static void fade_mixaudio(struct Channel *c,
         // Fading, but we have some space left in the current step.
         if (c->fade_off < c->fade_step_len) {
             int l = min(c->fade_step_len - c->fade_off, length);
-            mixaudio(dst, src, l, c->fade_vol);
 
+            mixaudio(dst, src, l, c->fade_vol);
+            
             length -= l;
             dst += l;
             src += l;
@@ -275,7 +331,7 @@ static void fade_mixaudio(struct Channel *c,
             c->fade_step_len = 0;
         }
     }
-
+    
     return;
 }
 
@@ -290,11 +346,62 @@ static void post_event(struct Channel *c) {
     SDL_PushEvent(&e);
 }
 
+/* This handels panning and vol2 manipulations. */
+static void pan_audio(struct Channel *c, Uint8 *stream, int length) {
+    int i;
+    short *sample = (short *) stream;
+    length /= 4;
+
+    float pan;
+    float vol2;
+    int left = 256;
+    int right = 256;
+    
+    
+    for (i = 0; i < length; i++) {
+
+        if ((i & 0x1f) == 0) {
+            pan = interpolate_pan(c);
+            vol2 = interpolate_vol2(c);
+            
+            // If nothing to do, skip 32 samples.
+            if (pan == 0.0 && vol2 == 1.0) {
+                i += 31;
+                c->pan_done += 32;
+                c->vol2_done += 32;
+                sample += 32 * 2;
+                continue;
+            }
+
+            vol2 *= 256.0;
+
+            if (pan < 0) {
+                left = (int) vol2;
+                right = (int) (vol2 * (1.0 + pan));
+            } else {
+                left = (int) (vol2 * (1.0 - pan));
+                right = (int) vol2;
+            }
+        }
+
+
+        *sample = (short) ((*sample * left) >> 8);
+        sample++;
+        *sample = (short) ((*sample * right) >> 8);
+        sample++;
+
+        c->pan_done += 1;
+        c->vol2_done += 1;
+    }
+    
+}
+
 static void callback(void *userdata, Uint8 *stream, int length) {
     int channel = 0;
-    
-    for (channel = 0; channel < NUM_CHANNELS; channel++) {
 
+    for (channel = 0; channel < num_channels; channel++) {
+
+        
         int mixed = 0;
         struct Channel *c = &channels[channel];
 
@@ -308,21 +415,22 @@ static void callback(void *userdata, Uint8 *stream, int length) {
 
         while (mixed < length && c->playing) {
             int mixleft = length - mixed;
-            int bufleft = c->decoded - c->last;
+            Uint8 buffer[mixleft];
 
+            // Decode some amount of data.
+
+            int bytes = ffpy_audio_decode(c->playing, buffer, mixleft);
+            
             // We have some data in the buffer.
-            if (c->stop_bytes && bufleft) {
-                int bytes = min(bufleft, mixleft);
-
+            if (c->stop_bytes && bytes) {
+                
                 if (c->stop_bytes != -1)
                     bytes = min(c->stop_bytes, bytes);
 
-                fade_mixaudio(c, &stream[mixed],
-                              &( ((Uint8*) c->playing->buffer) [c->last]),
-                              bytes);
+                pan_audio(c, buffer, bytes);
+                fade_mixaudio(c, &stream[mixed], buffer, bytes);
 
                 mixed += bytes;
-                c->last += bytes;
                 
                 if (c->stop_bytes != -1)
                     c->stop_bytes -= bytes;
@@ -335,13 +443,19 @@ static void callback(void *userdata, Uint8 *stream, int length) {
             // Otherwise, no data is left in the buffer. Check why,
             // and act accordingly.
 
-            // EOF or ERROR -- Skip to the next sample.
-            if (c->stop_bytes == 0 || c->playing->flags & (SOUND_SAMPLEFLAG_ERROR | SOUND_SAMPLEFLAG_EOF)) {
-                int old_tight = c->playing_tight;
+            // Skip to the next sample.
+            if (c->stop_bytes == 0 || bytes == 0) {
 
+                int old_tight = c->playing_tight;
+                struct Dying *d;
+                
                 post_event(c);
 
-                free_sample(c->playing);
+                d = malloc(sizeof(struct Dying));
+                d->next = dying;
+                d->stream = c->playing;
+                dying = d;
+                
                 decref(c->playing_name);
 
                 c->playing = c->queued;
@@ -355,31 +469,53 @@ static void callback(void *userdata, Uint8 *stream, int length) {
                 c->queued_tight = 0;
                 
                 start_sample(c, ! old_tight);
-/*                 update_pause(); */
                 
                 continue;                
             }
-
-            // If we're here, we're simply out of data in the
-            // buffer. Decode some.
-
-            c->decoded = Sound_Decode(c->playing);
-            c->last = 0;
         }
+
     }
 }
 
 /*
  * Checks that the given channel is in range. Returns 0 if it is,
- * sets an error and returns -1 if it is not.
+ * sets an error and returns -1 if it is not. Allocates channels
+ * that don't already exist.
  */
 static int check_channel(int c) {
-    if (c < 0 || c >= NUM_CHANNELS) {
+    int i;
+
+    if (c < 0) {
         error(PSS_ERROR);
         error_msg = "Channel number out of range.";
         return -1;
     }
 
+    if (c >= num_channels) {
+        channels = realloc(channels, sizeof(struct Channel) * (c + 1));
+    
+        for (i = num_channels; i <= c; i++) {
+         
+            channels[i].playing = NULL;
+            channels[i].queued = NULL;
+            channels[i].playing_name = NULL;
+            channels[i].queued_name = NULL;
+            channels[i].volume = SDL_MIX_MAXVOLUME;        
+            channels[i].paused = 1;
+            channels[i].event = 0;
+            channels[i].pan_start = 0.0;
+            channels[i].pan_end = 0.0;
+            channels[i].pan_length = 0;
+            channels[i].pan_done = 0;
+            channels[i].vol2_start = 1.0;
+            channels[i].vol2_end = 1.0;
+            channels[i].vol2_length = 0;
+            channels[i].vol2_done = 0;
+        }
+
+        num_channels = c + 1;
+    }
+    
     return 0;
 }
 
@@ -388,15 +524,9 @@ static int check_channel(int c) {
  * Loads the provided sample. Returns the sample on success, NULL on
  * failure.
  */
-static Sound_Sample *load_sample(SDL_RWops *rw, const char *ext) {
-    Sound_AudioInfo ai;
-    Sound_Sample *rv;
-    
-    ai.format = audio_spec.format;
-    ai.channels = audio_spec.channels;
-    ai.rate = audio_spec.freq;
-
-    rv = Sound_NewSample(rw, ext, &ai, audio_spec.samples * audio_spec.channels * 2);
+struct VideoState *load_sample(SDL_RWops *rw, const char *ext) {
+    struct VideoState *rv;    
+    rv = ffpy_stream_open(rw, ext);
     return rv;
 }
 
@@ -667,14 +797,20 @@ void PSS_fadeout(int channel, int ms) {
         return;
     }
 
+
+        
     fade_steps = c->volume;
     c->fade_delta = -1;
     c->fade_off = 0;
     c->fade_vol = c->volume;
 
-    c->fade_step_len = ms_to_bytes(ms) / fade_steps;
-    c->fade_step_len &= ~0x7; // Even sample.
-
+    if (fade_steps) {
+        c->fade_step_len = ms_to_bytes(ms) / fade_steps;
+        c->fade_step_len &= ~0x7; // Even sample.
+    } else {
+        c->fade_step_len = 0;
+    }
+        
     c->stop_bytes = ms_to_bytes(ms);
     c->queued_tight = 0;
 
@@ -719,7 +855,7 @@ void PSS_unpause_all(void) {
 
     ENTER();
 
-    for (i = 0; i < NUM_CHANNELS; i++) {
+    for (i = 0; i < num_channels; i++) {
         channels[i].paused = 0;
     }
 
@@ -805,6 +941,7 @@ void PSS_set_volume(int channel, float volume) {
 }
 
 
+
 float PSS_get_volume(int channel) {
 
     float rv;
@@ -828,13 +965,63 @@ float PSS_get_volume(int channel) {
     return rv;
 }
 
+/*
+ * This sets the pan of the channel... independent volumes for the
+ * left and right channels.
+ */
+void PSS_set_pan(int channel, float pan, float delay) {
+    struct Channel *c;
+    BEGIN();
+    
+    if (check_channel(channel)) {
+        return;
+    }
+
+    c = &channels[channel];
+
+    ENTER();
+    
+    c->pan_start = interpolate_pan(c);
+    c->pan_end = pan;
+    c->pan_length = (int) (audio_spec.freq * delay);
+    c->pan_done = 0;
+    
+    EXIT();
+    
+    error(SUCCESS);    
+}
+
+/*
+ * This sets the secondary volume of the channel.
+ */
+void PSS_set_secondary_volume(int channel, float vol2, float delay) {
+    struct Channel *c;
+    BEGIN();
+    
+    if (check_channel(channel)) {
+        return;
+    }
+
+    c = &channels[channel];
+
+    ENTER();
+
+    c->vol2_start = interpolate_vol2(c);
+    c->vol2_end = vol2;
+    c->vol2_length = (int) (audio_spec.freq * delay);
+    c->vol2_done = 0;
+    
+    EXIT();
+    
+    error(SUCCESS);    
+}
+
 
 /*
  * Initializes the sound to the given frequencies, channels, and
  * sample buffer size.
  */
-void PSS_init(int freq, int stereo, int samples) {
-    int i;
+void PSS_init(int freq, int stereo, int samples, int status) {
 
     if (initialized) {
         return;
@@ -865,31 +1052,15 @@ void PSS_init(int freq, int stereo, int samples) {
     audio_spec.callback = callback;
     audio_spec.userdata = NULL;
 
-    if (SDL_OpenAudio(&audio_spec, NULL)) {
+    if (SDL_OpenAudio(&audio_spec, &audio_spec)) {
         error(SDL_ERROR);
         return;
     }
 
-    if (! Sound_Init()) {
-        SDL_CloseAudio();
-        error(SOUND_ERROR);
-        return;
-    }
-
-    /*
-     * Initialize the channels.
-     */
-
-    for (i = 0; i < NUM_CHANNELS; i++) {
-        channels[i].playing = NULL;
-        channels[i].queued = NULL;
-        channels[i].volume = SDL_MIX_MAXVOLUME;        
-        channels[i].paused = 1;
-        channels[i].event = 0;
-    }
-
     SDL_PauseAudio(0);
     
+    ffpy_init(audio_spec.freq, status);
+
     initialized = 1;
 
     error(SUCCESS);
@@ -908,17 +1079,60 @@ void PSS_quit() {
     SDL_PauseAudio(1);
     EXIT();
 
-    for (i = 0; i < NUM_CHANNELS; i++) {
+    for (i = 0; i < num_channels; i++) {
         PSS_stop(i);
     }
 
-    Sound_Quit();
     SDL_CloseAudio();
 
+    num_channels = 0;
     initialized = 0;
     error(SUCCESS);
 }
 
+/* This must be called frequently, to take care of deallocating dead
+ * streams. */
+void PSS_periodic() {
+    BEGIN();
+
+    int i;
+
+    if (!dying) {
+        return;
+    }
+
+    ENTER();
+    
+    while (dying) {
+        struct Dying *d = dying;
+        ffpy_stream_close(d->stream);
+        dying = d->next;
+        free(d);
+    }
+    
+    EXIT();
+}
+
+/* This should be called in response to an FF_ALLOC_EVENT, with a pygame
+ * surface to display the movie on. */
+void PSS_alloc_event(PyObject *surface) {
+    int i;
+    for (i = 0; i < num_channels; i++) {
+        if (channels[i].playing) {
+            ffpy_alloc_event(channels[i].playing, surface);
+        }
+    }
+}
+
+/* This should be called in response to a FF_REFRESH_EVENT */
+void PSS_refresh_event(void) {
+    int i;
+    for (i = 0; i < num_channels; i++) {
+        if (channels[i].playing) {
+            ffpy_refresh_event(channels[i].playing);
+        }
+    }
+}
 
 /*
  * Returns the error message string if an error has occured, or
@@ -931,7 +1145,7 @@ const char *PSS_get_error() {
     case SDL_ERROR:
         return SDL_GetError();
     case SOUND_ERROR:
-        return Sound_GetError();
+        return "Some sort of ffmpeg error.";
     case PSS_ERROR:
         return error_msg;
     default:
