@@ -1,4 +1,4 @@
-# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -26,10 +26,10 @@
 
 import renpy
 
+import time
 import pygame
 import os
 import atexit
-import time
 
 # Import the appropriate modules, or set them to None if we cannot.
 
@@ -48,15 +48,18 @@ if 'pss' not in disable:
 
 if 'mix' not in disable:
     try:
-        import winmixer as mix
+        import winmixer as mix; mix
     except:
         try:
-            import linmixer as mix
+            import linmixer as mix; mix
         except:
             pass
 
 # Save the mixer, and restore it at exit.
 
+old_wave = None
+old_midi = None
+        
 if mix:
 
     mixer_enabled = False
@@ -139,6 +142,8 @@ class MusicContext(renpy.python.RevertableObject):
 
     def __init__(self):
 
+        super(MusicContext, self).__init__()
+        
         # The time this channel was last ordered panned.
         self.pan_time = None
 
@@ -283,7 +288,7 @@ class Channel(object):
 
         rv = mcd.get(self.name)
         if rv is None:
-           rv = mcd[self.name] = MusicContext();
+            rv = mcd[self.name] = MusicContext()
 
         return rv
 
@@ -368,6 +373,12 @@ class Channel(object):
                 self.playing = True
 
             except:
+
+                # If playing failed, remove topq.filename from self.loop
+                # so we don't keep trying.
+                while topq.filename in self.loop:
+                    self.loop.remove(topq.filename)
+
                 if renpy.config.debug_sound:
                     raise
                 else:
@@ -384,7 +395,7 @@ class Channel(object):
                 
         # Queue empty callback.
         if do_callback and self.callback:
-            self.callback()
+            self.callback() # E1102
 
     def dequeue(self, even_tight=False):
         """
@@ -424,11 +435,9 @@ class Channel(object):
                 pss.set_secondary_volume(self.number,
                                          self.context.secondary_volume,
                                          0)
-                
-        self.pan_delay = 0
             
         if not self.queue and self.callback:
-            self.callback()
+            self.callback() # E1102
 
 
     def fadeout(self, secs):
@@ -591,7 +600,6 @@ def init():
 def quit():
 
     global pcm_ok
-    global midi_ok
     global mix_ok
 
     if not pcm_ok:
@@ -779,7 +787,7 @@ def event(ev):
     if ev.type == REFRESH_EVENT:
         if renpy.audio.music.get_playing("movie"):
             pss.refresh_event()
-            
+
         # Return False, as a Movie should get this to know when to
         # redraw itself.
         return False
