@@ -26,7 +26,7 @@ import sys
 
 # Extra things used for distribution.
 def extra_imports():
-    import datetime
+    import datetime; datetime
     import encodings.ascii
     import encodings.utf_8
     import encodings.zlib_codec
@@ -35,29 +35,29 @@ def extra_imports():
     import encodings.raw_unicode_escape
     import encodings.mbcs
     import encodings.utf_16_be
-    import encodings.utf_16_le
-    import math
-    import glob
-    import pickle
-    import platform
+    import encodings.utf_16_le; encodings.utf_16_le
+    import math; math
+    import glob; glob
+    import pickle; pickle
+    import platform; platform
     import pysdlsound
     import pysdlsound.sound
     import pysdlsound.winmixer
-    import pysdlsound.linmixer
-    import difflib
-    import shutil
+    import pysdlsound.linmixer; pysdlsound.linmixer
+    import difflib; difflib
+    import shutil; shutil
     import renpy.tools.archiver
-    import renpy.tools.add_from
-    import tarfile
-    import bz2
-    import webbrowser
+    import renpy.tools.add_from; renpy.tools.add_from
+    import tarfile; tarfile
+    import bz2; bz2
+    import webbrowser; webbrowser
     import pygame.locals
     import pygame.color
-    import pygame.colordict
-    import posixpath
+    import pygame.colordict; pygame.colordict
+    import posixpath; posixpath # W0403
     import ctypes
-    import ctypes.wintypes
-    import EasyDialogs
+    import ctypes.wintypes; ctypes.wintypes
+    import EasyDialogs; EasyDialogs
     
 trace_file = None
 trace_local = None
@@ -90,15 +90,15 @@ def popen_del(self, _deadstate=sys.maxint):
     self.poll(_deadstate=_deadstate)
 
     try:
-        import subprocess
-        if self.returncode is None and subprocess._active is not None:
-            subprocess._active.append(self)
+        import subprocess # W0403
+        if self.returncode is None and subprocess._active is not None: # E1101
+            subprocess._active.append(self) # E1101
     except:
         pass
             
 def bootstrap(renpy_base):
 
-    global renpy
+    global renpy # W0602
 
     os.environ["RENPY_BASE"] = os.path.abspath(renpy_base)
     
@@ -226,12 +226,12 @@ def bootstrap(renpy_base):
 
     # Show the presplash.
     if not options.lint and not options.compile and not options.version and not options.rmpersistent:
-        import renpy.display.presplash
+        import renpy.display.presplash # W0404
         renpy.display.presplash.start(gamedir)
 
     # Fix an exception thrown by garbage collection.
-    import subprocess
-    subprocess.Popen.__del__ = popen_del
+    import subprocess # W0403
+    subprocess.Popen.__del__ = popen_del # E1101
 
     # If we're on a mac, install our own os.start.
     if sys.platform == "darwin":
@@ -266,7 +266,7 @@ def bootstrap(renpy_base):
             break
             
         except renpy.game.UtterRestartException:
-
+            
             # Only works after a full restart.
             report_error = renpy.display.error.ReportError()
             
@@ -294,6 +294,8 @@ def bootstrap(renpy_base):
     if options.leak:
         memory_profile()
 
+    renpy.display.im.cache.quit()
+    
     sys.exit(0)
 
 
@@ -311,6 +313,8 @@ def report_tb(out, tb):
         elif 'self' in f.f_locals:
             obj = f.f_locals['self']
 
+            import renpy
+            
             if isinstance(obj, renpy.execution.Context):
                 obj.report_tb(out)
 
@@ -325,12 +329,15 @@ def report_exception(e):
     type, value, tb = sys.exc_info()
 
     def safe_utf8(e):
-        m = e.message
-        
+        try:
+            m = unicode(e)
+        except:
+            m = str(e)
+            
         if isinstance(m, unicode):
             return m.encode("utf-8")
         else:
-            return str(m)
+            return m
 
     # Outside of the file.
     traceback.print_tb(tb, None, sys.stdout)
@@ -389,8 +396,6 @@ def report_exception(e):
 
 def memory_profile():
 
-    import renpy
-
     print "Memory Profile"
     print
     print "Showing all objects in memory at program termination."
