@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,7 +22,6 @@
 # This file contains support for state-machine controlled animations.
 
 import renpy
-import pygame
 import random
 
 class State(object):
@@ -77,7 +76,7 @@ class State(object):
     def motion_copy(self, child):
 
         if self.image is not None:
-            child = self.mage
+            child = self.image
 
         return State(self.name, child, *self.atlist)
     
@@ -279,7 +278,8 @@ class SMAnimation(renpy.display.core.Displayable):
 
             im = renpy.display.render.render(self.edge_cache, width, height, t - self.edge_start, at)
 
-            renpy.display.render.redraw(self.edge_cache, self.edge.delay - (t - self.edge_start))
+            if not renpy.game.less_updates:
+                renpy.display.render.redraw(self.edge_cache, self.edge.delay - (t - self.edge_start))
 
 
         iw, ih = im.get_size()
@@ -450,7 +450,8 @@ class TransitionAnimation(renpy.display.core.Displayable):
             
         for image, prev, delay, trans in zip(self.images, self.prev_images, self.delays, self.transitions):
             if t < delay:
-                renpy.display.render.redraw(self, delay - t)
+                if not renpy.game.less_updates:
+                    renpy.display.render.redraw(self, delay - t)
 
                 if trans and orig_t >= self.delays[0]:
                     image = trans(old_widget=prev, new_widget=image)
@@ -555,34 +556,14 @@ class Blink(renpy.display.core.Displayable):
 
 
         rend = renpy.display.render.render(self.image, height, width, st, at)
-
-        if not renpy.display.module.can_linmap:
-            return rend
-
         w, h = rend.get_size()
         rv = renpy.display.render.Render(w, h)
 
-        if alpha:
-
-            oldsurf = rend.pygame_surface()
-
-            if not (oldsurf.get_masks()[3]):
-                oldsurf = oldsurf.convert_alpha()
-
-            newsurf = pygame.Surface(oldsurf.get_size(), oldsurf.get_flags(), oldsurf)
-
-            # amap = renpy.display.im.ramp(0, int(alpha * 255.0))
-            # identity = renpy.display.im.identity
-
-            renpy.display.module.linmap(oldsurf, newsurf,
-                                        256, 256, 256, int(alpha * 256.0))
-
-            renpy.display.render.mutated_surface(newsurf)
-
-            rv.blit(newsurf, (0, 0))
-
-        rv.depends_on(rend)
-        renpy.display.render.redraw(self, delay)
+        rv.blit(rend, (0, 0))
+        rv.alpha = alpha
+        
+        if not renpy.game.less_updates:
+            renpy.display.render.redraw(self, delay)
 
         return rv
 
