@@ -1,4 +1,4 @@
-# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -26,7 +26,6 @@ import renpy
 from renpy.display.render import render
 
 import pygame
-from pygame.constants import *
 
 def compile_event(key, keydown):
     """
@@ -330,9 +329,11 @@ class SayBehavior(renpy.display.layout.Null):
     
 class Button(renpy.display.layout.Window):
 
+    keymap = { }
+    
     def __init__(self, child, style='button', clicked=None,
                  hovered=None, unhovered=None, role='',
-                 time_policy=None, 
+                 time_policy=None, keymap={},
                  **properties):
 
         super(Button, self).__init__(child, style=style, **properties)
@@ -343,7 +344,8 @@ class Button(renpy.display.layout.Window):
         self.unhovered = unhovered
         self.focusable = clicked is not None
         self.role = role
-
+        self.keymap = keymap
+        
         self.time_policy_data = None
         
         
@@ -393,31 +395,59 @@ class Button(renpy.display.layout.Window):
         if self.activated:
             return None
 
+        rv = None
+            
         if self.hovered and not default:
-            return self.hovered()
+            rv = self.hovered()
 
+        self.set_transform_event(self.role + "hover")
+        self.child.set_transform_event(self.role + "hover")
+        
+        return rv
+        
 
     def unfocus(self):
         super(Button, self).unfocus()
 
         if self.activated:
             return None
-        
+
         if self.unhovered:
             self.unhovered()
 
+        self.set_transform_event(self.role + "idle")
+        self.child.set_transform_event(self.role + "idle")
+        
+            
     def per_interact(self):
         if not self.clicked:
-            self.set_style_prefix(self.role + "insensitive_")
+            self.set_style_prefix(self.role + "insensitive_", True)
 
         super(Button, self).per_interact()
             
     def event(self, ev, x, y, st):
 
+        # If we have a child, try passing the event to it.
+        rv = super(Button, self).event(ev, x, y, st)
+        if rv is not None:
+            return rv
+        
         # If not focused, ignore all events.
         if not self.is_focused():
             return None
-        
+
+        # Check the keymap.
+        for name, action in self.keymap.iteritems():
+            if map_event(ev, name):
+                rv = action()
+                
+                if rv is not None:
+                    return rv
+                
+        # Ignore as appropriate:
+        if map_event(ev, "button_ignore") and self.clicked:
+            raise renpy.display.core.IgnoreEvent()
+            
         # If clicked, 
         if map_event(ev, "button_select") and self.clicked:
 
@@ -435,15 +465,20 @@ class Button(renpy.display.layout.Window):
                 self.activated = False
 
                 if self.is_focused():
-                    self.set_style_prefix(self.role + "hover_")
+                    self.set_style_prefix(self.role + "hover_", True)
                 else:
-                    self.set_style_prefix(self.role + "idle_")
+                    self.set_style_prefix(self.role + "idle_", True)
                     
                 raise renpy.display.core.IgnoreEvent()
                     
         return None
 
 
+    def set_style_prefix(self, prefix, root):
+        if root:
+            super(Button, self).set_style_prefix(prefix, root)
+    
+
 # Reimplementation of the TextButton widget as a Button and a Text
 # widget.
 def TextButton(text, style='button', text_style='button_text',
@@ -452,39 +487,108 @@ def TextButton(text, style='button', text_style='button_text',
     text = renpy.display.text.Text(text, style=text_style)
     return Button(text, style=style, clicked=clicked, **properties)
 
+
+# This is used for an input that takes its focus from a button. 
+class HoveredProxy(object):
+    def __init__(self, a, b):
+        self.a = a
+        self.b = b
+
+    def __call__(self):
+        self.a()
+        if self.b:
+            return self.b()
+    
                 
 class Input(renpy.display.text.Text):
     """
     This is a Displayable that takes text as input.
     """
 
+    changed = None
+    prefix = ""
+    suffix = ""
+    
     def __init__(self, default, length=None,
                  style='input_text',
                  allow=None,
                  exclude=None,
+                 prefix="",
+                 suffix="",
+                 changed=None,
+                 button=None,
                  **properties):
 
-        super(Input, self).__init__(default.replace("{", "{{") + "_", style=style, **properties)
+        super(Input, self).__init__("", style=style, **properties)
 
         self.content = unicode(default)
         self.length = length
 
         self.allow = allow
         self.exclude = exclude
+        self.prefix = prefix
+        self.suffix = suffix
 
+        self.changed = changed
+
+        self.editable = True
+
+        caretprops = { 'color' : None}
+        for i in properties:
+            if i.endswith("color"):
+                caretprops[i] = properties[i]
+
+        self.caret = renpy.display.image.Solid(xmaximum=1, style=style, **caretprops)
+
+        if button:
+            self.editable = False
+            button.hovered = HoveredProxy(self.enable, button.hovered)
+            button.unhovered = HoveredProxy(self.disable, button.unhovered)
+        
+        self.update_text(self.content, self.editable)
+        
+    def update_text(self, content, editable):
+
+        if content != self.content or editable != self.editable:
+            renpy.display.render.redraw(self, 0)
+                                            
+        if content != self.content:
+            self.content = content
+
+            if self.changed:
+                self.changed(content)
+                
+        self.editable = editable
+                                            
+        if editable:
+            self.set_text([self.prefix, content.replace("{", "{{"), self.suffix, self.caret])
+        else:
+            self.set_text([self.prefix, content.replace("{", "{{"), self.suffix ])
+
+    def enable(self):
+        self.update_text(self.content, True)
+
+    def disable(self):
+        self.update_text(self.content, False)
+            
     def event(self, ev, x, y, st):
 
+        if not self.editable:
+            return None
+        
         if map_event(ev, "input_backspace"):
-            if self.content:
-                self.content = self.content[:-1]
 
-            self.set_text(self.content.replace("{", "{{") + "_")
+            if self.content:
+                content = self.content[:-1]
+                self.update_text(content, self.editable)
+                                            
             renpy.display.render.redraw(self, 0)
 
         elif map_event(ev, "input_enter"):
-            return self.content
+            if not self.changed:
+                return self.content
 
-        elif ev.type == KEYDOWN and ev.unicode:
+        elif ev.type == pygame.KEYDOWN and ev.unicode:
             if ord(ev.unicode[0]) < 32:
                 return None
                 
@@ -497,10 +601,9 @@ class Input(renpy.display.text.Text):
             if self.exclude and ev.unicode in self.exclude:
                 raise renpy.display.core.IgnoreEvent()
 
-            self.content += ev.unicode
+            content = self.content + ev.unicode
 
-            self.set_text(self.content.replace("{", "{{") + "_")
-            renpy.display.render.redraw(self, 0)
+            self.update_text(content, self.editable)
 
             raise renpy.display.core.IgnoreEvent()
 
@@ -513,6 +616,8 @@ adj_registered = { }
 class Adjustment(renpy.object.Object):
 
     def __init__(self, range=1, value=0, step=None, page=0, changed=None, adjustable=True, ranged=None):
+        super(Adjustment, self).__init__()
+
         self._value = value
         self._range = range
         self._page = page
@@ -595,14 +700,14 @@ class Bar(renpy.display.core.Displayable):
 
     __version__ = 1
 
-    def after_upgrade(version):
+    def after_upgrade(self, version):
 
         if version < 1:
-            self.adjustment = Adjustment(self.range, self.value, changed=self.changed)
+            self.adjustment = Adjustment(self.range, self.value, changed=self.changed) # E1101
             self.adjustment.register(self)
-            del self.range
-            del self.value
-            del self.changed
+            del self.range # E1101
+            del self.value # E1101 
+            del self.changed # E1101
     
     def __init__(self,
                  range=None,
@@ -630,6 +735,12 @@ class Bar(renpy.display.core.Displayable):
 
         self.adjustment = adjustment
         self.focusable = adjustment.adjustable
+
+        # These are set when we are first rendered.
+        self.thumb_dim = 0
+        self.height = 0
+        self.width = 0
+        self.hidden = False
         
     def visit(self):
         return [ self.style.fore_bar, self.style.aft_bar, self.style.thumb, self.style.thumb_shadow ]
@@ -652,7 +763,7 @@ class Bar(renpy.display.core.Displayable):
                 self.hidden = True
                 return renpy.display.render.Render(width, height)
             elif self.style.unscrollable == "insensitive":
-                self.set_style_prefix("insensitive_")
+                self.set_style_prefix("insensitive_", True)
 
         self.hidden = False
         
@@ -745,8 +856,18 @@ class Bar(renpy.display.core.Displayable):
             rv.add_focus(self, None, 0, 0, width, height)
 
         return rv
+
+    
+    def focus(self, default=False):
+        super(Bar, self).focus(default)
+        self.set_transform_event("hover")
+
+        
+    def unfocus(self):
+        super(Bar, self).unfocus()
+        self.set_transform_event("idle")
+        
     
-      
     def event(self, ev, x, y, st):
 
         if not self.focusable:
@@ -782,7 +903,7 @@ class Bar(renpy.display.core.Displayable):
             if map_event(ev, "bar_increase"):
                 value += self.adjustment.step
 
-            if ev.type in (MOUSEMOTION, MOUSEBUTTONUP, MOUSEBUTTONDOWN):
+            if ev.type in (pygame.MOUSEMOTION, pygame.MOUSEBUTTONUP, pygame.MOUSEBUTTONDOWN):
 
                 if self.style.bar_vertical:
 
@@ -822,7 +943,8 @@ class Bar(renpy.display.core.Displayable):
             return self.adjustment.change(value)
 
         return None
-     
+
+
 class Conditional(renpy.display.layout.Container):
     """
     This class renders its child if and only if the condition is
