@@ -1,4 +1,4 @@
-# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -25,7 +25,6 @@
 import renpy
 
 import pygame
-from pygame.constants import *
 import sys
 import os
 import time
@@ -44,10 +43,13 @@ cpu_idle = threading.Event()
 cpu_idle.clear()
 
 # Need to be +4, so we don't interfere with FFMPEG's events.
-TIMEEVENT = USEREVENT + 4
-PERIODIC = USEREVENT + 5
-JOYEVENT = USEREVENT + 6
-REDRAW = USEREVENT + 7
+TIMEEVENT = pygame.USEREVENT + 4
+PERIODIC = pygame.USEREVENT + 5
+JOYEVENT = pygame.USEREVENT + 6
+REDRAW = pygame.USEREVENT + 7
+
+# All events except for TIMEEVENT and REDRAW
+ALL_EVENTS = [ i for i in range(0, REDRAW + 1) if i != TIMEEVENT and i != REDRAW ]
 
 # The number of msec between periodic events.
 PERIODIC_INTERVAL = 50
@@ -91,8 +93,14 @@ class Displayable(renpy.object.Object):
     focusable = False
     full_focus_name = None
     role = ''
+
+    # The event we'll pass on to our parent transform.
+    transform_event = None
+
+    # Can we change our look in response to transform_events?
+    transform_event_responder = False
     
-    def __init__(self, focus=None, default=False, style='default', **properties):
+    def __init__(self, focus=None, default=False, style='default', **properties): # W0231
         self.style = renpy.style.Style(style, properties, heavy=True)
         self.focus_name = focus
         self.default = default
@@ -110,13 +118,14 @@ class Displayable(renpy.object.Object):
 
             i.find_focusable(callback, focus_name)
 
+            
     def focus(self, default=False):
         """
         Called to indicate that this widget has the focus.
         """
 
         if not self.activated:
-            self.set_style_prefix(self.role + "hover_")
+            self.set_style_prefix(self.role + "hover_", True)
         
         if not default and not self.activated:
             if self.style.sound:
@@ -128,23 +137,27 @@ class Displayable(renpy.object.Object):
         """
 
         if not self.activated:
-            self.set_style_prefix(self.role + "idle_")
+            self.set_style_prefix(self.role + "idle_", True)
 
     def is_focused(self):
+
+        if renpy.display.focus.grab and renpy.display.focus.grab is not self:
+            return
+
         return renpy.game.context().scene_lists.focused is self
 
-    def set_style_prefix(self, prefix):
+    def set_style_prefix(self, prefix, root):
         """
         Called to set the style prefix of this widget and its child
         widgets, if any.
+
+        `root` - True if this is the root of a style tree, False if this
+        hass been passed on to a child.
         """
 
         if prefix == self.style.prefix:
             return
         
-        # if prefix == self.style_prefix:
-        #    return
-
         self.style.set_prefix(prefix)
         renpy.display.render.redraw(self, 0)
 
@@ -208,7 +221,7 @@ class Displayable(renpy.object.Object):
 
         while worklist:
             d = worklist.pop(0)
-
+            
             if d is None:
                 continue
 
@@ -324,6 +337,7 @@ class Displayable(renpy.object.Object):
         xoff += xoffset
         yoff += yoffset
 
+
         if subpixel:
             dest.subpixel_blit(surf, (xoff, yoff), main=main)
         else:
@@ -331,6 +345,43 @@ class Displayable(renpy.object.Object):
             
         return xoff, yoff
 
+    def set_transform_event(self, event):
+        """
+        Sets the transform event of this displayable to event.
+        """
+
+        if event == self.transform_event:
+            return
+
+        self.transform_event = event
+        if self.transform_event_responder:
+            renpy.display.render.redraw(self, 0)
+
+    def hide(self, st, at):
+        """
+        Returns None if this displayable is ready to be hidden, or
+        a replacement displayable if it doesn't want to be hidden
+        quite yet.
+        """
+                
+        return None
+
+    def show(self):
+        """
+        Called when the displayable is added to a scene list.
+        """
+
+    def get_parameterized(self):
+        """
+        If this is a ImageReference to a parameterized image, return
+        the get_parameterized() of the parameterized image. Otherwise,
+        return this displayable.
+        """
+
+        return self
+        
+        
+
 class ImagePredictInfo(renpy.object.Object):
     """
     This stores information involved in image prediction.
@@ -342,6 +393,8 @@ class ImagePredictInfo(renpy.object.Object):
     
     def __init__(self, ipi=None):
 
+        super(ImagePredictInfo, self).__init__()
+        
         # layer -> (tag -> image name)
         self.images = { }
 
@@ -382,7 +435,7 @@ class SceneLists(renpy.object.Object):
     things to the user. 
     """
 
-    __version__ = 2
+    __version__ = 3
     
     def after_setstate(self):
         for i in renpy.config.layers + renpy.config.top_layers:
@@ -401,16 +454,24 @@ class SceneLists(renpy.object.Object):
                 self.at_list[i] = { }
                 self.layer_at_list[i] = (None, [ ])
 
+        if version < 3:
+            self.shown_window = False 
+                
+                
     def __init__(self, oldsl, ipi):
 
+        super(SceneLists, self).__init__()
+        
+        # Has a window been shown as part of these scene lists?
+        self.shown_window = False
+        
         # A map from layer name -> list of
         # (key, zorder, show time, animation time, displayable) 
         self.layers = { }
         self.at_list = { }
         self.layer_at_list = { }
-        
         self.image_predict_info = ipi
-        
+
         if oldsl:
 
             for i in renpy.config.layers + renpy.config.top_layers:
@@ -447,10 +508,7 @@ class SceneLists(renpy.object.Object):
         """
 
         for i in renpy.config.transient_layers:
-            self.layers[i] = [ ]
-            self.at_list[i].clear()
-            self.image_predict_info.images[i].clear()
-            self.layer_at_list[i] = (None, [ ])
+            self.clear(i, True)
             
     def transient_is_empty(self):
         """
@@ -467,7 +525,28 @@ class SceneLists(renpy.object.Object):
 
         return True
 
-    def add(self, layer, thing, key=None, zorder=0, behind=[ ], at_list=[ ], name=None):
+    def transform_state(self, old_thing, new_thing):
+        """
+        If the old thing is a transform, then move the state of that transform
+        to the new thing.
+        """
+
+        if old_thing is None:
+            return new_thing
+        
+        old_transform = old_thing.get_parameterized()
+        if not isinstance(old_transform, renpy.display.motion.Transform):
+            return new_thing
+
+        new_transform = new_thing.get_parameterized()
+        if not isinstance(new_transform, renpy.display.motion.Transform):
+            new_thing = new_transform = renpy.display.motion.Transform(child=new_thing)
+        
+        new_transform.take_state(old_transform)
+        return new_thing
+
+                        
+    def add(self, layer, thing, key=None, zorder=0, behind=[ ], at_list=[ ], name=None, atl=None, default_transform=None):
         """
         This is called to add something to a layer. Layer is
         the name of the layer that we need to add the thing to,
@@ -503,18 +582,44 @@ class SceneLists(renpy.object.Object):
         at = None
         st = None
 
+        if atl:
+            thing = renpy.display.motion.ATLTransform(atl, child=thing)
+
         if key is not None:
+
+            hidekey = "hide$" + key
+            
             for index, (k, zo, st, at, d) in enumerate(l):
                 if k == key:
                     break
+
+                # If we're adding something with the same name as something
+                # that's hiding, remove the hiding thing. 
+                if k == hidekey:
+                    l.pop(index)
+                    index = None
+                    at = None
+                
             else:
                 index = None
                 at = None
 
+                
             st = None
 
             if index is not None:
-                if zorder == zo:                
+
+                old_thing = l[index][4]
+                
+                # If the old thing was a transform, make sure the new thing
+                # is a transform, and then take the transform state.
+
+                thing = self.transform_state(old_thing, thing)
+                    
+                thing.set_transform_event("replace")
+                thing.show() 
+                    
+                if zorder == zo: # W0631
                     l[index] = (key, zorder, st, at, thing)
                     return
                 else:
@@ -530,8 +635,12 @@ class SceneLists(renpy.object.Object):
         else:
             index = len(l)
 
+        thing = self.transform_state(default_transform, thing)
+            
+        thing.set_transform_event("show")
+        thing.show()
         l.insert(index, (key, zorder, st, at, thing))
-
+        
     def remove(self, layer, thing):
         """
         Thing is either a key or a displayable. This iterates through the
@@ -546,22 +655,67 @@ class SceneLists(renpy.object.Object):
             raise Exception("Trying to remove something from non-existent layer '%s'." % layer)
         
         l = self.layers[layer]
-        l = [ (k, zo, st, at, d) for k, zo, st, at, d in l if k != thing if d is not thing ]
-        self.layers[layer] = l
+        newl = [ ]
+
+        now = get_time()
+        
+        for i in l:
+            k, zo, st, at, d = i
+
+            if k == thing or d is thing:
+
+                # Should we keep this around while hiding it?
+                if k:
+                    st = st or now
+                    at = at or now
+
+                    d = d.hide(now - st, now - at)
+
+                    if d is not None:
+                        k = "hide$" + k
+                        newl.append((k, zo, st, at, d))
+
+                continue
+
+            newl.append(i)
+
+        self.layers[layer] = newl
 
         self.at_list[layer].pop(thing, None)
         self.image_predict_info.images[layer].pop(thing, None)
             
 
-    def clear(self, layer):
+    def clear(self, layer, hide=False):
         """
         Clears the named layer, making it empty.
+
+        If hide is True, then objects are hidden. Otherwise, they are
+        totally wiped out.
         """
 
-        if layer not in self.layers is None:
-            raise Exception("Trying to clear non-existent layer '%s'." % layer)
+        now = get_time()
+        
+        l = self.layers[layer]
+        newl = [ ]
 
-        self.layers[layer] = [ ]
+        for i in l:
+            k, zo, st, at, d = i
+
+            # Should we keep this around while hiding it?
+            if hide and k:
+
+                st = st or now
+                at = at or now
+
+                d = d.hide(now - st, now - at)
+
+                if d is not None:
+                    k = "hide$" + k
+                    newl.append((k, zo, st, at, d))
+
+                continue
+
+        self.layers[layer] = newl
         self.at_list[layer].clear()
         self.image_predict_info.images[layer].clear()
         self.layer_at_list[layer] = (None, [ ])
@@ -586,7 +740,7 @@ class SceneLists(renpy.object.Object):
                 ll.append((k, zo, st or time, at or time, d))
 
             l[:] = ll
-            
+
 
     def showing(self, layer, name):
         """
@@ -608,7 +762,11 @@ class SceneLists(renpy.object.Object):
 
         if at_list:
             for a in at_list:
-                rv = a(rv)
+                
+                if isinstance(a, renpy.display.motion.Transform):
+                    rv = a(child=rv)
+                else:
+                    rv = a(rv)
 
                 f = renpy.display.layout.MultiBox(layout='fixed')
                 f.add(rv, time, time)
@@ -616,8 +774,36 @@ class SceneLists(renpy.object.Object):
 
         rv.layer_name = layer
         return rv
-    
 
+    def remove_hidden(self):
+        """
+        Goes through all of the layers, and removes things that are
+        hidden and are no longer being kept alive by their hide
+        methods.
+        """
+
+        now = get_time()
+        
+        for l in self.layers:
+            newl = [ ]
+
+            for i in self.layers[l]:
+                name, zo, st, at, d = i
+
+                if name and name.startswith("hide$") and st and at:
+                    d = d.hide(now - st, now - at)
+
+                    if d is None:
+                        continue
+                    
+                    i = (name, zo, st, at, d)
+
+                newl.append(i)
+
+            self.layers[l] = newl
+
+
+            
 class Display(object):
     """
     This is responsible for managing the display window.
@@ -626,10 +812,6 @@ class Display(object):
 
     @ivar window: The window that is being presented to the user.
 
-    @ivar sample_surface: A sample surface that is optimized for fast
-    blitting to the window, with alpha. Used to create other surfaces
-    from.
-
     @ivar fullscreen: Is the window in fullscreen mode?
 
     @ivar mouse: The mouse image, if we have one, or None if
@@ -653,7 +835,6 @@ class Display(object):
 
     """
 
-
     def __init__(self, interface):
 
         self.interface = interface
@@ -700,7 +881,7 @@ class Display(object):
             # Convert the aspect ratio to be square.
             iw, ih = im.get_size()
             imax = max(iw, ih)
-            square_im = renpy.display.scale.PygameSurface((imax, imax), pygame.SRCALPHA, depth=32)
+            square_im = renpy.display.pgrender.surface_unscaled((imax, imax), True)
             square_im.blit(im, ( (imax-iw)/2, (imax-ih)/2 ))
             im = square_im
 
@@ -715,7 +896,7 @@ class Display(object):
         # If we're in fullscreen mode, and changing to another mode, go to
         # windowed mode first.
         s = pygame.display.get_surface()
-        if s and (s.get_flags() & FULLSCREEN):
+        if s and (s.get_flags() & pygame.FULLSCREEN):
             fullscreen = False
             
         self.fullscreen = fullscreen
@@ -748,37 +929,31 @@ class Display(object):
                 if w < renpy.config.screen_width or h < renpy.config.screen_height:
                     continue
                 
-                fsflag = FULLSCREEN
+                fsflag = pygame.FULLSCREEN
                 width = w
                 height = h
                 self.screen_xoffset = (width - renpy.config.screen_width) / 2
                 self.screen_yoffset = (height - renpy.config.screen_height) / 2
 
         elif fullscreen:
-            fsflag = FULLSCREEN
+            fsflag = pygame.FULLSCREEN
               
         # If a window exists of the right size and flags, use it. Otherwise,
         # make our own window.
         old_window = pygame.display.get_surface()
         if ((old_window is not None) and 
             (old_window.get_size() == (width, height)) and
-            (old_window.get_flags() & FULLSCREEN == fsflag)):
+            (old_window.get_flags() & pygame.FULLSCREEN == fsflag)):
             
             self.window = old_window
                     
         else:
-            self.window = pygame.display.set_mode((width, height), fsflag, 32)
-
-                
+            self.window = renpy.display.pgrender.set_mode((width, height), fsflag, 32)
 
         # Window title.
         self.window_caption = None
         self.set_window_caption()
         
-        # Sample surface that all surfaces are created based on.
-        sample = pygame.Surface((10, 10), 0, self.window)
-        self.sample_surface = sample.convert_alpha(self.window)
-
         pygame.event.set_grab(False)
         
         # Load the mouse image, if any.
@@ -810,6 +985,7 @@ class Display(object):
         # Setup periodic event.
         pygame.time.set_timer(PERIODIC, PERIODIC_INTERVAL)
 
+        
     def set_window_caption(self):
         caption = renpy.config.window_title + renpy.store._window_subtitle
         if caption == self.window_caption:
@@ -818,8 +994,10 @@ class Display(object):
         self.window_caption = caption
         pygame.display.set_caption(caption.encode("utf-8"))
 
+        
     def iconify(self):
         pygame.display.iconify()
+
         
     def can_redraw(self, first_pass):
         """
@@ -854,11 +1032,13 @@ class Display(object):
 
         return True
 
+    
     def mouse_event(self, ev):
 
         if ev.type == pygame.MOUSEMOTION or pygame.MOUSEBUTTONDOWN or pygame.MOUSEBUTTONUP:
             self.mouse_event_time = get_time()
-         
+
+            
     def show_mouse(self, pos, info):
         """
         Actually shows the mouse.
@@ -878,13 +1058,14 @@ class Display(object):
         by = my - myo
 
         self.mouse_backing_pos = (bx, by)
-        self.mouse_backing = pygame.Surface((mw, mh), self.window.get_flags(), self.window)
+        self.mouse_backing = renpy.display.pgrender.surface((mw, mh), False)
         self.mouse_backing.blit(self.window, (0, 0), (bx, by, mw, mh))
 
         self.window.blit(mouse, (bx, by))
 
         return bx, by, mw, mh
 
+    
     def hide_mouse(self):
         """
         Actually hides the mouse.
@@ -901,6 +1082,7 @@ class Display(object):
 
         return rv
 
+    
     def draw_mouse(self, show_mouse=True):
         """
         This draws the mouse to the screen, if necessary. It uses the
@@ -961,6 +1143,7 @@ class Display(object):
             
         return updates
 
+    
     def update_mouse(self):
         """
         Draws the mouse, and then updates the screen.
@@ -1007,6 +1190,7 @@ class Display(object):
         
         self.suppressed_blit = suppress_blit
         self.surftree = surftree
+
         
     def save_screenshot(self, filename):
         """
@@ -1019,7 +1203,6 @@ class Display(object):
             if renpy.config.debug:
                 raise
             pass
-        
             
         
     def screenshot(self, scale):
@@ -1027,9 +1210,9 @@ class Display(object):
         Returns a string containing the contents of the window, as a PNG.
         """
 
-        surf = self.window.convert_alpha()
+        surf = renpy.display.pgrender.copy_surface(self.window, True)
         surf = renpy.display.scale.smoothscale(surf, scale)
-        surf = surf.convert()
+        surf = renpy.display.pgrender.copy_surface(surf, False)
         
         sio = cStringIO.StringIO()
         renpy.display.module.save_png(surf, sio, 0)
@@ -1112,6 +1295,7 @@ class Interface(object):
         self.timeout_time = None
         self.last_event = None
         self.current_context = None
+        self.roll_forward = None
         
         # Should we reset the display?
         self.display_reset = False
@@ -1158,7 +1342,10 @@ class Interface(object):
         # for contexts outside the current one. This is used to restore those
         # in the case where nothing has changed in the new context.
         self.transition_info_stack = [ ]
-                
+
+        # The time when the event was dispatched.
+        self.event_time = 0
+        
                 
     def take_screenshot(self, scale):
         """
@@ -1168,6 +1355,7 @@ class Interface(object):
 
         self.screenshot = self.display.screenshot(scale)
 
+        
     def get_screenshot(self):
         """
         Gets the current screenshot, as a string. Returns None if there isn't
@@ -1179,6 +1367,7 @@ class Interface(object):
 
         return self.screenshot
 
+    
     def lose_screenshot(self):
         """
         This deallocates the saved screenshot.
@@ -1186,17 +1375,19 @@ class Interface(object):
 
         self.screenshot = None
 
+        
     def show_window(self):
 
         if not renpy.store._window:
             return
 
-        if self.shown_window:
+        if renpy.game.context().scene_lists.shown_window:
             return
 
         if renpy.config.empty_window:
             renpy.config.empty_window()
 
+            
     def do_with(self, trans, paired, clear=False):
         
         if renpy.config.with_callback:
@@ -1218,21 +1409,25 @@ class Interface(object):
         be transitioning from.
         """
 
+        # Show the window, if that's necessary.
         self.show_window()
-        
+
+        # Compute the overlay.
+        self.compute_overlay()
+
         scene_lists = renpy.game.context().scene_lists
 
-        old_old_scene = self.old_scene
-
+        # Compute the scene.
         self.old_scene = self.compute_scene(scene_lists)        
 
-        if renpy.config.overlay_during_with and old_old_scene:
-            for i in renpy.config.overlay_layers:
-                self.old_scene[i] = old_old_scene[i]
+        # Get rid of transient things.
+
+        for i in renpy.config.overlay_layers:
+            scene_lists.clear(i)
 
-        scene_lists = renpy.game.context().scene_lists
         scene_lists.replace_transient()
-        self.shown_window = False
+        scene_lists.shown_window = False
+
         
     def set_transition(self, transition, layer=None, force=False):
         """
@@ -1256,7 +1451,9 @@ class Interface(object):
 
         ev = pygame.event.poll()
 
-        if ev.type == NOEVENT:
+        if ev.type == pygame.NOEVENT:
+            # Seems to prevent the CPU from speeding up.
+            time.sleep(0.001)
             return None
 
         self.pushed_event = ev
@@ -1299,7 +1496,25 @@ class Interface(object):
         self.last_event = ev
         return ev
 
+    
+    def compute_overlay(self):
 
+        if renpy.store.suppress_overlay:
+            return
+        
+        # Figure out what the overlay layer should look like.
+        renpy.ui.layer("overlay")
+
+        for i in renpy.config.overlay_functions:
+            i()
+
+        if renpy.game.context().scene_lists.shown_window:
+            for i in renpy.config.window_overlay_functions:
+                i()
+                
+        renpy.ui.close()
+        
+    
     def compute_scene(self, scene_lists):
         """
         This converts scene lists into a dictionary mapping layer
@@ -1334,6 +1549,7 @@ class Interface(object):
         else:                
             raise renpy.game.QuitException()
 
+        
     def interact(self, clear=True, suppress_window=False, **kwargs):
         """
         This handles an interaction, restarting it if necessary. All of the
@@ -1344,17 +1560,15 @@ class Interface(object):
         if not suppress_window:
             self.show_window()
         
+
         # These things can be done once per interaction.
 
-        import time
-        start = time.time()
-        
         preloads = self.preloads
         self.preloads = [ ]
 
         try:
             renpy.game.after_rollback = False
-            
+
             for i in renpy.config.start_interact_callbacks:
                 i()
 
@@ -1378,8 +1592,10 @@ class Interface(object):
                 
             self.restart_interaction = True
 
-            self.shown_window = False
+            renpy.game.context().scene_lists.shown_window = False
             
+
+    
     def interact_core(self,
                       show_mouse=True,
                       trans_pause=False,
@@ -1404,7 +1620,8 @@ class Interface(object):
         @param suppress_underlay: This suppresses the display of the underlay.
         """
         
-        suppress_overlay = suppress_overlay or renpy.store.suppress_overlay        
+        self.roll_forward = roll_forward
+        
         suppress_transition = renpy.config.skipping or renpy.game.less_updates
 
         # The global one.
@@ -1462,8 +1679,10 @@ class Interface(object):
         renpy.display.layout.size_groups.clear()
         
         # Clear some events.
-        pygame.event.clear((MOUSEMOTION, PERIODIC,
-                            TIMEEVENT, REDRAW))
+        pygame.event.clear((pygame.MOUSEMOTION,
+                            PERIODIC,
+                            TIMEEVENT,
+                            REDRAW))
 
         # Add a single TIMEEVENT to the queue.
         pygame.event.post(self.time_event)
@@ -1471,14 +1690,12 @@ class Interface(object):
         # Figure out the scene list we want to show.        
         scene_lists = renpy.game.context().scene_lists
 
-        # Figure out what the overlay layer should look like.
-        renpy.ui.layer("overlay")
+        # Remove the now-hidden things.
+        scene_lists.remove_hidden()
 
+        # Compute the overlay.
         if not suppress_overlay:
-            for i in renpy.config.overlay_functions:
-                i()
-
-        renpy.ui.close()
+            self.compute_overlay()
 
         # The root widget of everything that is displayed on the screen.
         root_widget = renpy.display.layout.MultiBox(layout='fixed') 
@@ -1548,11 +1765,7 @@ class Interface(object):
             old_root.layers = { }
 
             for layer in renpy.config.layers:
-                if layer in renpy.config.overlay_layers:
-                    d = scene[layer]
-                else:                
-                    d = self.transition_from[None].layers[layer]
-
+                d = self.transition_from[None].layers[layer]
                 old_root.layers[layer] = d
                 old_root.add(d)
             
@@ -1562,9 +1775,10 @@ class Interface(object):
 
             if not isinstance(trans, Displayable):
                 raise Exception("Expected transition to be a displayable, not a %r" % trans)
+
+            trans.show()
             
             transition_time = self.transition_time.get(None, None)
-
             root_widget.add(trans, transition_time, transition_time)
 
             if trans_pause:
@@ -1646,7 +1860,9 @@ class Interface(object):
                     self.force_redraw = False
 
                 # Redraw the screen.
-                if needs_redraw and self.display.can_redraw(first_pass):
+                if (needs_redraw and
+                    (first_pass or not pygame.event.peek(ALL_EVENTS)) and 
+                    self.display.can_redraw(first_pass)):
                     
                     # If we have a movie, start showing it.
                     suppress_blit = renpy.display.video.interact()
@@ -1761,7 +1977,6 @@ class Interface(object):
                             if self.timeout_time != old_timeout_time:
                                 # Always set to at least 1ms.
                                 pygame.time.set_timer(TIMEEVENT, int(time_left * 1000 + 1))
-
                                 old_timeout_time = self.timeout_time
 
                     # Handle autosaving, as necessary.
@@ -1776,7 +1991,7 @@ class Interface(object):
                         else:
                             ev = self.event_wait()
 
-                    if ev.type == NOEVENT:
+                    if ev.type == pygame.NOEVENT:
                         continue
 
                     if renpy.config.profile:
@@ -1790,7 +2005,7 @@ class Interface(object):
 
                         ev2 = self.event_peek()
 
-                        if ev2 and ev2.type not in (NOEVENT, PERIODIC, REDRAW, QUIT):
+                        if ev2 and ev2.type not in (pygame.NOEVENT, PERIODIC, REDRAW, pygame.QUIT):
                             ev = self.event_poll()
                             
                     # Handle redraw timeouts.
@@ -1827,12 +2042,12 @@ class Interface(object):
                     renpy.display.behavior.skipping(ev)
                     
                     # Handle quit specially for now.
-                    if ev.type == QUIT:
+                    if ev.type == pygame.QUIT:
                         self.quit_event()
 
                     # Merge mousemotion events.
-                    if ev.type == MOUSEMOTION:
-                        evs = pygame.event.get([MOUSEMOTION])
+                    if ev.type == pygame.MOUSEMOTION:
+                        evs = pygame.event.get([pygame.MOUSEMOTION])
                         if len(evs):
                             ev = evs[-1]
 
@@ -1840,12 +2055,11 @@ class Interface(object):
                             self.focused = True
                             
                     # Handle focus notifications.
-                    if ev.type == ACTIVEEVENT:
+                    if ev.type == pygame.ACTIVEEVENT:
                         if ev.state & 1:
                             self.focused = ev.gain
                             
-                    # x, y = getattr(ev, 'pos', (0, 0))
-                    x, y = pygame.mouse.get_pos()
+                    x, y = getattr(ev, 'pos', pygame.mouse.get_pos())
                     x -= self.display.screen_xoffset
                     y -= self.display.screen_yoffset
                     
