@@ -1,4 +1,4 @@
-# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,9 +21,8 @@
 
 # This file contains code to manage focus on the display.
 
-import renpy
 import pygame
-from pygame.constants import *
+import renpy
 
 class Focus(object):
 
@@ -169,18 +168,14 @@ def before_interact(roots):
     for f, n in fwn:
         if f is not current:
             f.unfocus()
-
+            
     if current:
         current.focus(default=True)
-
     
 
 # This changes the focus to be the widget contained inside the new
 # focus object.
 def change_focus(newfocus, default=False):
-
-    global argument
-
     rv = None
     
     if grab:
@@ -217,7 +212,7 @@ def change_focus(newfocus, default=False):
 # This handles mouse events, to see if they change the focus.
 def mouse_handler(ev, x, y, default=False):
 
-    if ev.type not in (MOUSEMOTION, MOUSEBUTTONUP, MOUSEBUTTONDOWN):
+    if ev.type not in (pygame.MOUSEMOTION, pygame.MOUSEBUTTONUP, pygame.MOUSEBUTTONDOWN):
         return
 
     new_focus = renpy.display.render.focus_at_point(x, y)
