@@ -1,4 +1,4 @@
-# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -73,7 +73,7 @@ class Minigame(renpy.display.core.Displayable):
 
         if rv is None:
 
-            font = renpy.display.text.get_font(font, size, bold=bold, italics=italics, underline=underline)
+            font = renpy.display.text.get_font(font, size, bold, italics, underline)
             rv = font.render(text, antialias, color)
             self.mutated_surface(rv)
 
