@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,13 +19,15 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-import pygame
-from pygame.constants import *
-
 import re
 import renpy
 import sys
 
+from _renpybidi import log2vis, WRTL, RTL, ON
+    
+get_font = renpy.display.font.get_font
+ImageFont = renpy.display.font.ImageFont
+
 # A list of text tags, mapping from the text tag prefix to if it
 # requires a closing tag.
 text_tags = dict(
@@ -48,236 +50,6 @@ text_tags = dict(
 
 text_tags[""] = True
        
-# This contains a map from (fn, size, bold, italics, underline) to the
-# unloaded font object corresponding to that specification. 
-fonts = { }
-
-# This contains a map from (fn, size, bold, italics, underline) to the
-# loaded font object corresponding to that specification.
-font_cache = { }
-
-class SFont(object):
-
-    def __init__(self,
-                 filename,
-                 spacewidth,
-                 default_kern,
-                 kerns,
-                 charset):
-
-        self.filename = filename
-        self.spacewidth = spacewidth
-        self.default_kern = default_kern
-        self.kerns = kerns
-        self.charset = charset
-
-    def load(self):
-
-        # Map from character to subsurface.
-        self.chars = { }
-
-        # Map from character to width, height.
-        self.sizes = { }
-
-        # Load in the image.
-        surf = renpy.display.im.Image(self.filename).load(unscaled=True)
-        self.surf = surf
-
-        sw, sh = surf.get_size()
-        height = sh - 1
-        self.height = height
-
-        # Create space characters.
-        self.chars[u' '] = pygame.Surface((self.spacewidth, height), 0, surf)
-        self.sizes[u' '] = self.spacewidth
-        self.chars[u'\u00a0'] = self.chars[u' ']
-        self.sizes[u'\u00a0'] = self.sizes[u' ']
-
-        # The color key used to separate characters.
-        i = 0
-        while True:
-            key = surf.get_at((i, 0))
-            if key[3] != 0:
-                break
-            i += 1
-
-        ci = 0
-
-        # Find real characters, create them.
-        while i < sw and ci < len(self.charset):
-
-            if surf.get_at((i, 0)) != key:
-                start = i
-                i += 1
-
-                while i < sw:
-                    if surf.get_at((i, 0)) == key:
-                        break
-
-                    i += 1
-
-                c = self.charset[ci]
-                ci += 1
-
-                ss = surf.subsurface((start, 1, i - start, height))
-                ss = renpy.display.scale.surface_scale(ss)
-                
-                self.chars[c] = ss
-                self.sizes[c] = i - start
-
-
-            i += 1
-
-    def size(self, text):
-        w = 0
-
-        if not text:
-            return (0, self.height)
-        
-        for a, b in zip(text, text[1:]):
-            w += self.sizes[a] + self.kerns.get(a + b, self.default_kern)
-
-        w += self.sizes[text[-1]]
-
-        return (w, self.height)
-            
-    def render(self, text, antialias, color, black_color=(0, 0, 0, 255), background=None):
-        surf = pygame.Surface(self.size(text), 0,
-                              renpy.game.interface.display.sample_surface)
-
-        if not text:
-            return surf
-
-        x = 0
-        y = 0
-        
-        for a, b in zip(text, text[1:]):
-            surf.blit(self.chars[a], (x, y))
-            x += self.sizes[a] + self.kerns.get(a + b, self.default_kern)
-
-        surf.blit(self.chars[text[-1]], (x, y))
-
-        if renpy.config.recolor_sfonts and \
-               (color != (255, 255, 255, 255) or black_color != (0, 0, 0, 255) ) and \
-               renpy.display.module.can_twomap:
-
-            newsurf = pygame.Surface(surf.get_size(), surf.get_flags(), surf)
-            renpy.display.module.twomap(surf, newsurf, color, black_color)
-            renpy.display.render.mutated_surface(newsurf)
-
-            surf = newsurf
-
-        return surf
-
-    def get_linesize(self):
-        return self.height 
-
-    def get_height(self):
-        return self.height
-
-    def get_ascent(self):
-        return self.height
-
-    def get_descent(self):
-        return 0
-
-def register_sfont(name=None, size=None, bold=False, italics=False, underline=False, 
-                   filename=None, spacewidth=10, default_kern=0, kerns={},
-                   charset=u"!\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"):
-    """
-    This registers an SFont with the given details. Please note that
-    size, bold, italic, and underline are all advisory (used for matching), and
-    do not change the appearance of the font.
-
-    @param name: The name of the font being registered.
-    @param size: The size of the font being registered.
-    @param bold: The boldness of the font being registered.
-    @param italics: The italicness of the font being registered.
-    @param underline: The underline of the font being registered.
-
-    @param filename: The file containing the sfont image.
-    @param spacewidth: The width of a space character.
-    @param default_kern: The default kern spacing between characters.
-    @param kerns: A map from two-character strings to the kern that should
-    be used between those characters.
-    @param charset: The character set of the font. A string containing characters
-    in the order in which they are found in the image.
-    """
-   
-    if name is None or size is None or filename is None:
-        raise Exception("When registering an SFont, the font name, font size, and filename are required.")
-
-    sf = SFont(filename, spacewidth, default_kern, kerns, charset)
-    fonts[(name, size, bold, italics, underline)] = sf
-
-
-
-def load_ttf(fn, size, bold, italics, underline):
-
-    try:
-        rv = pygame.font.Font(renpy.loader.load(fn), size)
-        rv.set_bold(bold)
-        rv.set_italic(italics)
-    except:
-        # Let's try to find the font on our own.
-        fonts = [ i.strip().lower() for i in fn.split(",") ]
-
-        pygame.sysfont.initsysfonts()
-
-        rv = None
-
-        for k, v in pygame.sysfont.Sysfonts.iteritems():
-            for flags, ffn in v.iteritems():
-                for i in fonts:
-                    if ffn.lower().endswith(i):
-                        rv = pygame.font.Font(ffn, size)
-                        rv.set_bold(bold)
-                        rv.set_italic(italics)
-                        break
-                if rv:
-                    break
-            if rv:
-                break
-        else:
-            # Let pygame try to find the font for us.
-            rv = pygame.font.SysFont(fn, size, bold, italics)
-
-    rv.set_underline(underline)
-    return rv
-    
-
-# TODO: Something sane if the font file can't be found.
-def get_font(fn, size, bold=False, italics=False, underline=False):
-    from renpy.loader import transfn
-
-    if (fn, bold, italics) in renpy.config.font_replacement_map:
-        fn, bold, italics = renpy.config.font_replacement_map[fn, bold, italics]
-
-    if (fn, size, bold, italics, underline) in font_cache:
-        return font_cache[(fn, size, bold, italics, underline)]
-
-    rv = fonts.get((fn, size, bold, italics, underline), None)
-    if rv is not None:
-        rv.load()
-    else:
-        try:
-            rv = load_ttf(fn, size, bold, italics, underline)
-        except:
-            if renpy.config.debug:
-                raise
-            raise Exception("Could not find font: %r" % ((fn, size, bold, italics, underline), ))
-
-    font_cache[(fn, size, bold, italics, underline)] = rv
-
-    return rv
-
-def free_memory():
-    """
-    Clears the font cache.
-    """
-
-    font_cache.clear()
-
 def color(s):
     """
     This function converts a hexcode into a color/alpha tuple. Leading
@@ -327,12 +99,28 @@ class TextStyle(object):
     def __init__(self, source=None):
         if source is not None:
             vars(self).update(vars(source))
+        else:
+            self.font = ""
+            self.size = 0
+            self.bold = False
+            self.italic = False
+            self.underline = False
+            self.strikethrough = False
+            self.color = None
+            self.black_color = None
+            self.hyperlink = None
 
+            # The cached font.
+        self.f = None
+            
         # Width cache.
         self.wcache = { }
+
+        # Surface cache.
+        self.scache = { }
         
     def update(self):
-        self.f = get_font(self.font, self.size, self.bold, self.italic, self.underline)
+        self.f = get_font(self.font, self.size, self.bold, self.italic, self.underline, 0)
 
     def get_font(self):
         return self.f
@@ -349,18 +137,26 @@ class TextStyle(object):
         return rv
 
     def sizes(self, text):
+
         return self.get_width(text), self.f.get_ascent() - self.f.get_descent()
 
-    def render(self, text, antialias, color, black_color, use_colors, time, at):
-
+    def render(self, text, antialias, color, black_color, use_colors, time, at, expand, use_cache):
+        
         if use_colors:
             color = self.color or color
             black_color = self.black_color or black_color
+
+        key = (text, antialias, color, black_color, expand)
+        
+        if use_cache and key in self.scache:
+            return self.scache[key]
             
-        font = self.f
-
-
-        if isinstance(font, SFont):
+        if expand:
+            font = get_font(self.font, self.size, self.bold, self.italic, self.underline, expand)
+        else:
+            font = self.f
+            
+        if isinstance(font, ImageFont):
             rv = font.render(text, antialias, color, black_color)
             
             if self.strikethrough:
@@ -373,25 +169,21 @@ class TextStyle(object):
             color = (r, g, b, 255)
 
             surf = font.render(text, antialias, color)
-
+            rv = renpy.display.pgrender.copy_surface(surf)
+            
             if self.strikethrough:
                 sw, sh = surf.get_size()
                 soh = max(sh / 10, 1) 
-                surf.subsurface((0, sh / 2, sw, soh)).fill(color) 
+                rv.subsurface((0, sh / 2, sw, soh)).fill(color) 
 
-            if a != 255 and renpy.display.module.can_linmap:
-
-                if not surf.get_masks()[3]:
-                    surf = surf.convert_alpha()
-
-                rv = pygame.Surface(surf.get_size(), surf.get_flags(), surf)
-
-                renpy.display.module.linmap(surf, rv, 256, 256, 256, a + 1)
-            
-            else:
-                rv = surf
+            if a != 255:
+                renpy.display.module.linmap(rv, rv, 256, 256, 256, a + 1)
 
         renpy.display.render.mutated_surface(rv)
+        
+        if use_cache and not renpy.game.less_memory:
+            self.scache[key] = (rv, rv.get_size())
+        
         return rv, rv.get_size()
 
     def length(self, text):
@@ -428,7 +220,7 @@ class WidgetStyle(object):
     def sizes(self, widget):
         return self.width, self.height
 
-    def render(self, widget, antialias, color, black_color, foreground, st, at):
+    def render(self, widget, antialias, color, black_color, foreground, st, at, expand, use_cache):
 
         # If in the foreground
         if foreground:
@@ -439,7 +231,23 @@ class WidgetStyle(object):
     def length(self, text):
         return 1
 
+class SpacerStyle(object):
 
+    def update(self):
+        pass
+    
+    # The actual size is computed at render time.
+    def sizes(self, ignored):
+        return (0, 0)
+
+    def get_ascent(self):
+        return 0
+
+    def get_width(self, text):
+        return 0
+
+    def length(self, text):
+        return 0
     
 # The line breaking algorithm for western languages.    
 western_text_regexp = re.compile(ur"""(?x)
@@ -552,28 +360,48 @@ def input_tokenizer(l, style, pauses=None):
     return rv
 
 
-def layout_width(triples):
+def layout_width(triples, justify=False):
     """
-    Returns the width of the given list of triples. Cache
-    should be a dictionary, which is used to cache results.
+    Returns the width of the given list of triples. 
     """
 
     rv = 0
 
     curts = None
-    cur = None
-            
+    cur = ""
+
+    rtl = renpy.config.rtl
+    
     for type, ts, i in triples:
         if ts is not curts:
-            if curts:
+            if cur:
+
+                if rtl:
+                    cur, dir = log2vis(cur, ON)
+
                 rv += curts.get_width(cur)
+                
             curts = ts
             cur = i
+
+        elif justify and type == "space":
+            cur += i
+
+            if rtl:
+                cur, dir = log2vis(cur, ON)
+
+            rv += curts.get_width(cur)
+            cur = ""
+
         else:
             cur += i
 
     if curts:
+        if rtl:
+            cur, dir = log2vis(cur, ON)
+
         rv += curts.get_width(cur)
+
     return rv
     
 
@@ -588,11 +416,19 @@ def greedy_text_layout(triples, width, style):
 
     @param style: The style of the text widget.
     """
+
     
     lines = [ ]
     line = [ ]
+    
+    # Is this the last line in a paragraph?
+    lines_last = [ ]    
 
     target = width - style.first_indent
+
+    justify = style.justify
+    
+    after_newline = True
     
     for triple in triples:
 
@@ -600,43 +436,50 @@ def greedy_text_layout(triples, width, style):
 
         if type == "newline":
             lines.append(line)
-            line = [ triple ]
+            lines_last.append(True) 
+            line = [ ]
             target = width - style.rest_indent
 
+            after_newline = True
+            
             continue
 
         if type == "space":
-            if not line and lines:
+            if not line and not after_newline:
                 continue
             line.append(triple)
             continue
 
         else:
-            if layout_width(line + [ triple ]) > target:
+
+            after_newline = False
+            
+            if layout_width(line + [ triple ], justify) > target:
+
                 lines.append(line)
+                lines_last.append(False)
                 line = [ triple ]
                 target = width - style.rest_indent
             else:
                 line.append(triple)
-                
-    lines.append(line)
 
+    lines.append(line)
+    lines_last.append(True)
+    
     # Remove trailing whitespace, except on the last line, where it
     # might be intentional.
     for l in lines[:-1]:
         if l and l[-1][0] == "space":
             l.pop()
 
-    return lines
+    return lines, lines_last
 
 
-def subtitle_text_layout_core(triples, width, style, soft, n):
-
-    sizecache = { }
+def subtitle_text_layout_core(triples, width, style, soft, n, justify):
 
     lines = [ ]
     line = [ ]
-
+    
     target = width
 
     total = soft * n
@@ -654,7 +497,7 @@ def subtitle_text_layout_core(triples, width, style, soft, n):
 
         else:
 
-            lw = layout_width(line + [ triple ])
+            lw = layout_width(line + [ triple ], justify)
 
             if lw > target or type == "newline":
 
@@ -664,7 +507,7 @@ def subtitle_text_layout_core(triples, width, style, soft, n):
                     if line and line[-1][0] == "space":
                         line.pop()
 
-                    total -= layout_width(line)
+                    total -= layout_width(line, justify)
                     linesoft = total / n
                 else:
                     linesoft = soft
@@ -709,6 +552,8 @@ def subtitle_text_layout_core(triples, width, style, soft, n):
 
 def subtitle_text_layout(triples, width, style):
 
+    justify = style.justify
+    
     softwidth = style.subtitle_width
     if isinstance(softwidth, float):
         softwidth = int(softwidth * width)
@@ -725,25 +570,40 @@ def subtitle_text_layout(triples, width, style):
 
 
     # Deal with each paragraph separately.
-    rrv = [ ] # Real return value.
+    lines = [ ]
+    lines_last = [ ]
         
     for triples in pars:
+        x = 0
+        newtriples = []
+        while x<len(triples):
+            triplewidth = layout_width([triples[x],],justify)
+            if triplewidth<=width or triples[x][0]!='word':
+                newtriples.append(triples[x])
+            else:
+                for char in triples[x][2]:
+                    newtriples.append(('word',triples[x][1],char))
+            x+=1
+        triples = newtriples
 
-        sumwidths = layout_width(triples)
+        sumwidths = layout_width(triples, justify)
 
         i = 1
         while sumwidths / i > min(width, softwidth):
             i += 1
 
         while True:
-            rv = subtitle_text_layout_core(triples, width, style, sumwidths / i, i)
+            rv = subtitle_text_layout_core(triples, width, style, sumwidths / i, i, justify)
             if len(rv) == i:
                 break
             i += 1
-
-        rrv.extend(rv)
             
-    return rrv
+        lines.extend(rv)
+        for i in range(len(lines) - 1):
+            lines_last.append(False)
+        lines_last.append(True)
+                
+    return lines, lines_last
 
 def text_layout(triples, width, style):
 
@@ -760,12 +620,12 @@ class Text(renpy.display.core.Displayable):
 
     nosave = [ 'laidout', 'laidout_lineheights', 'laidout_linewidths',
                'laidout_width', 'laidout_height', 'laidout_start',
-               'laidout_length', 'laidout_hyperlinks', 'width', 'tokens', 'children',
-               'child_pos']
+               'laidout_length', 'laidout_hyperlinks', 'laidout_lines_last',
+               'width', 'tokens', 'children', 'child_pos']
 
     __version__ = 2
 
-    def after_upgrade(version):
+    def after_upgrade(self, version):
         if version <= 0:
             self.activated = None
         if version <= 1:
@@ -820,7 +680,12 @@ class Text(renpy.display.core.Displayable):
 
         self.tokens = None
 
+        # The duration of each pause, or None if the pause is infinite.
+        self.pause_lengths = [ ]
         
+        # The width we've been laid out for.
+        self.width = -1
+
         self.update(redraw=False)
 
 
@@ -868,29 +733,49 @@ class Text(renpy.display.core.Displayable):
 
         new_tokens = [ ]
         fasts = 0
-        self.no_wait = False
-        self.no_wait_once = False
-        self.no_wait_done = False
 
-        self.pauses = 0
+        self.no_wait = False # W0201
+        self.no_wait_once = False # W0201
+        self.no_wait_done = False # W0201
+        self.pauses = 0 # W0201
+        self.pause_lengths = [ ] # W0201
         
         for i in self.tokens[0]:
             type, text = i
             
             if type == "tag":
-                if text == "p" or text.startswith("p="):
-                    new_tokens.append(("tag", 'w' + text[1:]))
+                if text == "p":
+                    new_tokens.append(("tag", 'w'))
                     new_tokens.append(("newline", "\n"))
                     self.pauses += 1
+                    self.pause_lengths.append(None)
+
                     continue
+
+                elif text.startswith("p="):
+                    new_tokens.append(("tag", 'w=' + text[2:]))
+                    new_tokens.append(("newline", "\n"))
+                    self.pauses += 1
+                    self.pause_lengths.append(float(text[2:]))
+
+                    continue
+
                 elif text == "nw":
                     self.no_wait = True
+
                 elif text == "fast":
                     self.no_wait = False
                     fasts += 1
                     self.pauses = 0
-                elif text == "w" or text.startswith("w="):
+                    self.pause_lengths = [ ]
+                    
+                elif text == "w":
                     self.pauses += 1
+                    self.pause_lengths.append(None)
+
+                elif text.startswith("w="):
+                    self.pauses += 1
+                    self.pause_lengths.append(float(text[2:]))
                     
             new_tokens.append(i)
 
@@ -944,7 +829,7 @@ class Text(renpy.display.core.Displayable):
             
         # Postprocess the tokens list to create widgets, as necessary.
 
-        self.children = [ ]
+        self.children = [ ] # W0201
         newtokens = [ ]
         
         for tl in self.tokens:
@@ -954,7 +839,6 @@ class Text(renpy.display.core.Displayable):
 
             for kind, i in tliter:
 
-
                 if kind == "tag" and i.startswith("image="):
 
                     m = re.match(r'image=(.*)', i)
@@ -962,7 +846,7 @@ class Text(renpy.display.core.Displayable):
                     if not m:
                         raise Exception('Image tag %s could not be parsed.' % i)
 
-                    i = renpy.display.im.image(m.group(1))
+                    i = renpy.easy.displayable(m.group(1))
                     ntl.append(("widget", i))
                     self.children.append(i)
 
@@ -990,6 +874,7 @@ class Text(renpy.display.core.Displayable):
                 return True
             else:
                 renpy.game.interface.timeout((self.slow_done_time + self.pause_length) - st)
+
         if self.slow and self.style.slow_abortable and renpy.display.behavior.map_event(ev, "dismiss"):
             self.slow = False
             raise renpy.display.core.IgnoreEvent()
@@ -1020,10 +905,10 @@ class Text(renpy.display.core.Displayable):
             
             
     def visit(self):
-       if self.tokens is None:
+        if self.tokens is None:
             self.update()
 
-       return self.children
+        return self.children
 
     
     def layout(self, width, time):
@@ -1061,12 +946,12 @@ class Text(renpy.display.core.Displayable):
         tsl[-1].hyperlink = None
         tsl[-1].update()
 
-        self.laidout_hyperlinks = [ ]
+        self.laidout_hyperlinks = [ ] # W0201
         
-        if not self.text:
-            text = " "
-        else:
-            text = self.text
+        # if not self.text:
+        #     text = " "
+        # else:
+        #     text = self.text
 
         # for i in re.split(r'( |\{[^{}]+\}|\{\{|\n)', text):
 
@@ -1257,7 +1142,6 @@ class Text(renpy.display.core.Displayable):
                 raise Exception("Unknown text token kind %s." % kind)
 
             if kind == "widget":
-
                 wstyle = WidgetStyle(tsl[-1], i, width, time)
                 triples.append(("word", wstyle, i))
                             
@@ -1269,25 +1153,28 @@ class Text(renpy.display.core.Displayable):
 
         # Give text_layout a list of triples, get back a list of lists of
         # triples, one per line.
-        linetriples = renpy.config.text_layout(triples, width, self.style)
-
-
+        linetriples, lines_last = renpy.config.text_layout(triples, width, self.style)
+        
         # Now, we need to go through these lines, to generate the data
         # we need to render text.
 
-        self.laidout = [ ]
-        self.laidout_lineheights = [ ]
-        self.laidout_linewidths = [ ]
-        self.laidout_length = 0
-        self.laidout_start = 0
-        self.laidout_width = self.style.min_width
-        self.laidout_height = 0
-
+        self.laidout = [ ] # W0201
+        self.laidout_lineheights = [ ] # W0201
+        self.laidout_linewidths = [ ] # W0201
+        self.laidout_length = 0 # W0201
+        self.laidout_start = 0 # W0201
+        self.laidout_width = self.style.min_width # W0201
+        self.laidout_height = 0 # W0201
+        self.laidout_lines_last = lines_last # W0201
+        
         # Add something to empty lines.
         for l in linetriples:
             if not l:
                 l.append(('word', tsl[-1], ' '))
 
+
+        justify = self.style.justify
+                
         for l in linetriples:
 
             line = [ ]
@@ -1304,7 +1191,7 @@ class Text(renpy.display.core.Displayable):
                     self.laidout_length += len(i)
                 except:
                     self.laidout_length += ts.length(i)
-
+                    
                 if ts is not oldts:
                     if oldts is not None:
                         line.append((oldts, cur))
@@ -1314,9 +1201,34 @@ class Text(renpy.display.core.Displayable):
                 else:
                     cur += i
 
+                if justify and kind == "space":
+                    if cur:
+                        line.append((oldts, cur))
+                        cur = ""
+                    line.append((SpacerStyle(), ""))
+                    
             if oldts:
                 line.append((oldts, cur))
 
+                
+            if renpy.config.rtl:
+
+                rtl_line = [ ]
+                
+                # RTL direction.
+                line_direction = ON
+
+                for ts, i in line:
+                    if isinstance(ts, TextStyle):
+                        i, line_direction = log2vis(i, line_direction)
+
+                    rtl_line.append((ts, i))
+
+                if line_direction == RTL or line_direction == WRTL:
+                    rtl_line.reverse()
+
+                line = rtl_line
+                
             width = 0
             height = 0
 
@@ -1376,14 +1288,13 @@ class Text(renpy.display.core.Displayable):
         """
 
         if not self.laidout:
-            import sys
             return sys.maxint
 
         return self.laidout_length
 
             
 
-    def render_pass(self, r, offsets, color, black_color, user_colors, length, time, at, child_pos, add_focus):
+    def render_pass(self, r, xo, yo, color, black_color, user_colors, length, time, at, child_pos, add_focus, expand):
         """
         Renders the text to r at the offsets. Color is the base color,
         and user_colors controls if the user can override those colors.
@@ -1397,40 +1308,65 @@ class Text(renpy.display.core.Displayable):
         rest_indent = self.style.rest_indent
         antialias = self.style.antialias
         line_spacing = self.style.line_spacing
-        
-        for line, line_height, line_width in zip(self.laidout, self.laidout_lineheights, self.laidout_linewidths):
-            x = indent + self.style.text_align * (self.laidout_width - line_width)
+        text_align = self.style.text_align
+        justify = self.style.justify
+
+        for line, line_height, line_width, last in zip(self.laidout, self.laidout_lineheights, self.laidout_linewidths, self.laidout_lines_last):
+            if justify and not last:
+                empty_space = (self.laidout_width - line_width - indent)
+                x = indent
+            else:
+                empty_space = 0
+                x = indent + text_align * (self.laidout_width - line_width)
+                
             indent = rest_indent
-
+                
             max_ascent = 0
-
+            spacers = 0
+                
             for ts, text in line:
                 max_ascent = max(ts.get_ascent(), max_ascent)
 
-            for ts, text in line:
+                if isinstance(ts, SpacerStyle):
+                    spacers += 1
                 
+            for ts, text in line:
+
+                if isinstance(ts, SpacerStyle):
+                    space = empty_space / spacers
+                    x += space
+                    empty_space -= space
+                    spacers -= 1
+                    continue
+                    
                 length -= ts.length(text)
 
+                # Should we cache the rendered text?
+                use_cache = True
+                
                 if length < 0:
+                    use_cache = False
+                    
                     if isinstance(text, (str, unicode)):
                         text = text[:length]
                     else:
                         return False
 
-                surf, (sw, sh) = ts.render(text, antialias, color, black_color, user_colors, time, at)
+                surf, (sw, sh) = ts.render(text, antialias, color, black_color, user_colors, time, at, expand, use_cache)
 
+                if expand:
+                    (sw, sh) = ts.sizes(text)
+                    
                 actual_y = y + max_ascent - ts.get_ascent()
 
-                for xo, yo in offsets:
+                if surf:
+                    r.blit(surf, (x + xo, actual_y + yo))
 
-                    if surf:
-                        r.blit(surf, (x + xo, actual_y + yo))
+                if add_focus and ts.hyperlink is not None:
+                    r.add_focus(self, ts.hyperlink, x + xo, y + yo, sw, sh)
 
-                    if add_focus and ts.hyperlink is not None:
-                        r.add_focus(self, ts.hyperlink, x + xo, y + yo, sw, sh)
-                        
-                    if not isinstance(text, (str, unicode)):
-                        child_pos.append((text, x + xo, actual_y + yo))
+                if not isinstance(text, (str, unicode)):
+                    child_pos.append((text, x + xo, actual_y + yo))
                 
                 x = x + sw
 
@@ -1448,6 +1384,9 @@ class Text(renpy.display.core.Displayable):
         Called to call slow_done, and also to update slow_done_time.
         """
 
+        if not self.slow:
+            return
+
         self.slow = False
         
         if self.slow_done:
@@ -1478,9 +1417,12 @@ class Text(renpy.display.core.Displayable):
             if speed:
                 speed *= self.style.slow_cps_multiplier
 
+        if renpy.game.less_updates:
+            speed = None
                 
         dslist = self.style.drop_shadow
-
+        outlines = self.style.outlines
+        
         if dslist is None:
             dslist = [ ]
         elif not isinstance(dslist, list):
@@ -1497,6 +1439,12 @@ class Text(renpy.display.core.Displayable):
             maxdsx = max(maxdsx, dsx)
             maxdsy = max(maxdsy, dsy)
 
+        for expand, color, dsx, dsy in outlines:
+            mindsx = min(mindsx, dsx - expand)
+            mindsy = min(mindsy, dsy - expand)
+            maxdsx = max(maxdsx, dsx + expand)
+            maxdsy = max(maxdsy, dsy + expand)
+            
         # minds{x,y} are negative (or 0), maxds{x,y} are positive (or 0).
             
         self.layout(width + mindsx - maxdsx, st)
@@ -1507,16 +1455,18 @@ class Text(renpy.display.core.Displayable):
         else:
             length = sys.maxint
             self.call_slow_done(st)
-                
+
         rv = renpy.display.render.Render(self.laidout_width - mindsx + maxdsx, self.laidout_height - mindsy + maxdsy)
 
-        if dslist:
-            dsoffsets = [ (dsxo - mindsx, dsyo - mindsy) for dsxo, dsyo in dslist ]
-            self.render_pass(rv, dsoffsets, self.style.drop_shadow_color, self.style.drop_shadow_color, False, length, st, at, [ ], False)
+        for dsxo, dsyo in dslist:
+            self.render_pass(rv, dsxo - mindsx, dsyo - mindsy, self.style.drop_shadow_color, self.style.drop_shadow_color, False, length, st, at, [ ], False, 0)
 
+        for expand, color, dsxo, dsyo in outlines:
+            self.render_pass(rv, dsxo - mindsx - expand, dsyo - mindsy - expand, color, color, False, length, st, at, [ ], False, expand)
+             
         self.child_pos = [ ]
 
-        if self.render_pass(rv, [ (-mindsx, -mindsy) ], self.style.color, self.style.black_color, True, length, st, at, self.child_pos, True):
+        if self.render_pass(rv, -mindsx, -mindsy, self.style.color, self.style.black_color, True, length, st, at, self.child_pos, True, 0):
             if self.slow:
                 self.call_slow_done(st)
 
