@@ -1,4 +1,4 @@
-# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -37,8 +37,6 @@
 
 import renpy
 from renpy.display.render import render
-import pygame
-from pygame.constants import *
 
 
 class Transition(renpy.display.core.Displayable):
@@ -55,12 +53,12 @@ class Transition(renpy.display.core.Displayable):
     def event(self, ev, x, y, st):
 
         if self.events or ev.type == renpy.display.core.TIMEEVENT:
-            return self.new_widget.event(ev, x, y, st)
+            return self.new_widget.event(ev, x, y, st) # E1101
         else:
             return None
 
     def visit(self):
-        return [ self.new_widget, self.old_widget ]
+        return [ self.new_widget, self.old_widget ] # E1101
 
 def null_render(d, width, height, st, at):
 
@@ -243,18 +241,10 @@ class Pixellate(Transition):
     steps in each direction.
     """
 
-    nosave = [ 'surface', 'surface_size' ]
-                   
-    surface = None
-    surface_size = None
-    
     def __init__(self, time, steps, old_widget=None, new_widget=None, **properties):
 
         time = float(time)
 
-        if not renpy.display.module.can_pixellate:
-            time = 0
-
         super(Pixellate, self).__init__(time, **properties)
 
         self.time = time
@@ -287,18 +277,17 @@ class Pixellate(Transition):
         rdr = render(visible, width, height, st, at)
 
         surf = rdr.pygame_surface(False)
-        
-        if surf.get_size() != self.surface_size:
-            self.surface_size = surf.get_size()
-            self.surface = pygame.Surface(self.surface_size, surf.get_flags(), surf)
+        size = surf.get_size()
+
+        newsurf = renpy.display.pgrender.surface(size, surf)
 
         px = 2 ** step
 
-        renpy.display.module.pixellate(surf, self.surface, px, px, px, px)
-        renpy.display.render.mutated_surface(self.surface)
+        renpy.display.module.pixellate(surf, newsurf, px, px, px, px)
+        renpy.display.render.mutated_surface(newsurf)
 
         rv = renpy.display.render.Render(rdr.width, rdr.height)
-        rv.blit(self.surface, (0, 0))
+        rv.blit(newsurf, (0, 0))
         rv.depends_on(rdr)
         
         renpy.display.render.redraw(self, 0)
@@ -336,7 +325,7 @@ class Dissolve(Transition):
         if renpy.game.less_updates:
             return null_render(self, width, height, st, at)
         
-        if st >= self.time or not renpy.display.module.can_blend:
+        if st >= self.time:
             self.events = True
             return render(self.new_widget, width, height, st, at)
 
@@ -363,20 +352,232 @@ class Dissolve(Transition):
 
             if w <= 0 or h <= 0:
                 return
-            
+
             renpy.display.module.blend(
                 bottom_surface.subsurface((-x, -y, w, h)),
                 top_surface.subsurface((-x, -y, w, h)),
                 dest.subsurface((0, 0, w, h)),
                 alpha)
+            
+        if self.alpha:
 
+            surf = renpy.display.pgrender.surface((width, height), True)
+            draw(surf, 0, 0)
+            renpy.display.render.mutated_surface(surf)
+
+            rv = renpy.display.render.Render(width, height)
+            rv.blit(surf, (0, 0))
+
+        else:
+            rv = renpy.display.render.Render(width, height, draw_func=draw, opaque=True)
+
+        rv.depends_on(top, True)
+        rv.depends_on(bottom)
+        return rv
+
+
+class ImageDissolve(Transition):
+    """
+    This dissolves the old scene into the new scene, using an image
+    to control the dissolve process.
+
+    A list of values is used to control this mapping. This list of
+    values consists 256 fully transparent values, a ramp (of a
+    specified number of steps) from full transparency to full opacity,
+    and 256 fully opaque values. A 256 entry window is slid over this
+    list, and the values found within are used to map the red channel
+    of the image onto the opacity of the new scene.
+
+    Basically, this means that while pixels come in first, black last,
+    and the ramp controls the sharpness of the transition.
+    
+    @param image: The image that will be used to control this
+    transition. The image should be the same size as the scene being
+    dissolved.
+
+    @param time: The amount of time the dissolve will take.
+
+    @param ramplen: The number of pixels of ramp to use. This defaults
+    to 8.
+
+    @param ramptype: Type of alpha ramp. Possible types are: linear, cube,
+    dcube, mcube. Default is linear. Non-linear types must have
+    ramplen >= 8. "cube": Ease in, sharp out. "dcube": Sharp in, sharp out.
+    "mcube": Ease in, ease out.
+
+    @param ramp: If given, this is expected to be a sequence of
+    integers in the range 0 to 255. This sequence explicitly gives the
+    ramp to be used. If given, this overrides ramplen and ramptype.
+
+    @param reverse: This reverses the ramp and the direction of the window
+    slide. When True, black pixels dissolve in first, and while pixels come
+    in last.    
+    """
+
+    __version__ = 1
+
+    def after_upgrade(self, version):
+        if version < 1:
+            self.alpha = False
+    
+    def generate_ramp(self, ramplen, ramptype, explicit_ramp, reverse):
+        """
+        Precomputes the ramp.
+        """
+
+        ramp = '\x00' * 256
+
+        if explicit_ramp is not None:
+
+            for i in explicit_ramp:
+                ramp += chr(i)
+
+        else:
+
+            if ramptype == 'cube':
+                # make sure ramplen is big enough, to avoid div-by-0 errors
+                # Not much point in nonlinear if the size is that small, anyway
+                if ramplen >= 8:
+                    table = []
+                    for i in range(ramplen):
+                        table.append(i * i * i)
+                    scale = max(table) / 255.0
+                    for i in range(ramplen):
+                        #print i, table[i], table[i] / scale
+                        ramp += chr(int(table[i] / scale))
+                else:
+                    ramptype = 'linear'
+
+            elif ramptype == 'dcube':
+                if ramplen >= 8:
+                    table = []
+                    for i in range(ramplen / 2 - ramplen, ramplen / 2):
+                        table.append(i * i * i)
+                    adj = abs(min(table))
+                    for i in range(len(table)):
+                        table[i] += adj
+                    scale = max(table) / 255.0
+                    #print "scale:", scale
+                    for i in range(ramplen):
+                        #print i, table[i], table[i] / scale
+                        ramp += chr(int(table[i] / scale))
+                else:
+                    ramptype = 'linear'
+
+            elif ramptype == 'mcube':
+                if ramplen >= 8:
+                    ramplen = (ramplen / 2) * 2 # make sure it's even
+                    table = []
+                    for i in range(ramplen / 2):
+                        table.append(i * i * i)
+                    adj = table[-1]
+                    tmptable = []
+                    for i in table:
+                        tmptable.append(i)
+                    for i in range(1, len(table) + 1):
+                        tmptable.append(abs(table[len(table) - i] - adj) + adj)
+                    table = tmptable
+                    scale = max(table) / 255.0
+                    #print "scale:", scale
+                    for i in range(ramplen):
+                        #print i, table[i], table[i] / scale
+                        ramp += chr(int(table[i] / scale))
+                else:
+                    ramptype = 'linear'
+
+            if ramptype == 'linear':
+                for i in range(ramplen):
+                    ramp += chr(255 * i / ramplen)
+
+        ramp += '\xff' * 256
+
+        if reverse:
+            ramp = list(ramp)
+            ramp.reverse()
+            ramp = ''.join(ramp)
+
+        return ramp
+        
+        
+    def __init__(self, image, time, ramplen=8, ramptype='linear', ramp=None, reverse=False, alpha=False,
+                 old_widget=None, new_widget=None, **properties):
+
+        super(ImageDissolve, self).__init__(time, **properties)
+
+        self.time = time
+        self.old_widget = old_widget
+        self.new_widget = new_widget
+        self.events = False
+        self.alpha = alpha
+        
+        self.image = renpy.display.im.image(image)
+
+        if ramp is not None:
+            ramplen = len(ramp)
+
+        self.ramp = self.generate_ramp(ramplen, ramptype, ramp, reverse)
+        
+        self.steps = ramplen + 256
+        self.reverse = reverse
+
+
+    def visit(self):
+        return super(ImageDissolve, self).visit() + [ self.image ]
+
+    def render(self, width, height, st, at):
+
+        if renpy.game.less_updates:
+            return null_render(self, width, height, st, at)
+
+        if st >= self.time:
+            self.events = True
+            return render(self.new_widget, width, height, st, at)
+
+        image = renpy.display.im.cache.get(self.image)
+
+        if st < self.time:
+            renpy.display.render.redraw(self, 0)
+
+        step = int(self.steps * st / self.time)
+
+        if self.reverse:
+            step = self.steps - step
+
+        ramp = self.ramp[step:step+256]
+
+
+        bottom = render(self.old_widget, width, height, st, at)
+        top = render(self.new_widget, width, height, st, at)
+        
+        bottom_surface = bottom.pygame_surface(self.alpha)
+        top_surface = top.pygame_surface(self.alpha)
+
+        iw, ih = image.get_size()
+
+        width = min(bottom.width, top.width, iw)
+        height = min(bottom.height, top.height, ih)
+            
+        def draw(dest, x, y):
+
+            dw, dh = dest.get_size()
+
+            w = min(dw, width + x)
+            h = min(dh, height + y)
+
+            renpy.display.module.imageblend(
+                bottom_surface.subsurface((-x, -y, w, h)),
+                top_surface.subsurface((-x, -y, w, h)),
+                dest.subsurface((0, 0, w, h)),
+                image.subsurface((-x, -y, w, h)),
+                ramp)
 
         if self.alpha:
 
-            rv = renpy.display.render.Render(width, height)
-            surf = pygame.Surface((width, height), bottom_surface.get_flags(), bottom_surface)
+            surf = renpy.display.pgrender.surface((width, height), True)
             draw(surf, 0, 0)
             renpy.display.render.mutated_surface(surf)
+
+            rv = renpy.display.render.Render(width, height)
             rv.blit(surf, (0, 0))
 
         else:
@@ -760,9 +961,8 @@ def MoveTransition(delay, old_widget=None,  new_widget=None, factory=None, enter
 
                 if (isinstance(f, renpy.display.layout.MultiBox) 
                     and layer in layers
-                    and f.scene_list is not None 
-                    and layer not in renpy.config.overlay_layers):
-                        
+                    and f.scene_list is not None):
+                    
                     f = merge_slide(old.layers[layer], new.layers[layer])
 
                 rv.layers[layer] = f
@@ -773,35 +973,42 @@ def MoveTransition(delay, old_widget=None,  new_widget=None, factory=None, enter
         # Otherwise, we recompute the scene list for the two widgets, merging
         # as appropriate.
 
-        def tag_d(sle):
+        # Wraps the displayable found in SLE so that the various timebases
+        # are maintained.
+        def wrap(sle):
+            key, zorder, st, at, d = sle
+            return renpy.display.layout.AdjustTimes(d, st, at)
+        
+        def tag(sle):
             if sle[0] is None:
                 tag = sle[4]
             else:
                 tag = sle[0]
 
-            return tag, sle[4]
+            return tag
 
         def merge(sle, d):
             return (sle[0], sle[1], 0, sle[3], d)
         
         # A list of tags on the new layer.
-        new_tags = { }
+        new_tags = set()
 
         # The scene list we're creating.
         rv_sl = [ ]
 
         # The new scene list we're copying from.
         new_scene_list = new.scene_list[:]
-        
+
         for new_sle in new.scene_list:
-            new_tag, new_d = tag_d(new_sle)
+            new_tag = tag(new_sle)
             
             if new_tag is not None:
-                new_tags[new_tag] = new_d
+                new_tags.add(new_tag)
 
         for old_sle in old.scene_list:
-            old_tag, old_d = tag_d(old_sle)
-
+            old_tag = tag(old_sle)
+            old_d = wrap(old_sle)
+            
             # In old, not in new.
             if old_tag not in new_tags:
 
@@ -817,10 +1024,10 @@ def MoveTransition(delay, old_widget=None,  new_widget=None, factory=None, enter
             # In new, not in old.
             while new_scene_list:
                 new_sle = (new_scene_list.pop(0))                
-                new_tag, new_d = tag_d(new_sle)
-
-                if new_tag in new_tags:
-                    del new_tags[new_tag]
+                new_tag = tag(new_sle)
+                new_d = wrap(new_sle)
+                
+                new_tags.discard(new_tag)
                 
                 if new_tag == old_tag:
                     break
@@ -850,10 +1057,8 @@ def MoveTransition(delay, old_widget=None,  new_widget=None, factory=None, enter
         # scene list.
         while new_scene_list:
             new_sle = (new_scene_list.pop(0))                
-            new_tag, new_d = tag_d(new_sle)
-
-            if new_tag in new_tags:
-                del new_tags[new_tag]
+            new_tag = tag(new_sle)
+            new_d = wrap(new_sle)
 
             move = enter_factory(position(new_d), delay, new_d)
             if move is None:
@@ -862,8 +1067,6 @@ def MoveTransition(delay, old_widget=None,  new_widget=None, factory=None, enter
             rv_sl.append(merge(new_sle, move))
             continue
 
-        
-
         layer = new.layer_name
         rv = renpy.display.layout.MultiBox(layout='fixed', focus=layer, **renpy.game.interface.layer_properties[layer])
         rv.append_scene_list(rv_sl)
@@ -875,231 +1078,11 @@ def MoveTransition(delay, old_widget=None,  new_widget=None, factory=None, enter
     # This calls merge_slide to actually do the merging.
 
     rv = merge_slide(old_widget, new_widget)
-    rv.delay = delay
+    rv.delay = delay # W0201
 
     return rv
 
             
-class ImageDissolve(Transition):
-    """
-    This dissolves the old scene into the new scene, using an image
-    to control the dissolve process.
-
-    A list of values is used to control this mapping. This list of
-    values consists 256 fully transparent values, a ramp (of a
-    specified number of steps) from full transparency to full opacity,
-    and 256 fully opaque values. A 256 entry window is slid over this
-    list, and the values found within are used to map the red channel
-    of the image onto the opacity of the new scene.
-
-    Basically, this means that while pixels come in first, black last,
-    and the ramp controls the sharpness of the transition.
-    
-    @param image: The image that will be used to control this
-    transition. The image should be the same size as the scene being
-    dissolved.
-
-    @param time: The amount of time the dissolve will take.
-
-    @param ramplen: The number of pixels of ramp to use. This defaults
-    to 8.
-
-    @param ramptype: Type of alpha ramp. Possible types are: linear, cube,
-    dcube, mcube. Default is linear. Non-linear types must have
-    ramplen >= 8. "cube": Ease in, sharp out. "dcube": Sharp in, sharp out.
-    "mcube": Ease in, ease out.
-
-    @param ramp: If given, this is expected to be a sequence of
-    integers in the range 0 to 255. This sequence explicitly gives the
-    ramp to be used. If given, this overrides ramplen and ramptype.
-
-    @param reverse: This reverses the ramp and the direction of the window
-    slide. When True, black pixels dissolve in first, and while pixels come
-    in last.    
-    """
-
-    __version__ = 1
-
-    def after_upgrade(self, version):
-        if version < 1:
-            self.alpha = False
-    
-    def generate_ramp(self, ramplen, ramptype, explicit_ramp, reverse):
-        """
-        Precomputes the ramp.
-        """
-
-        ramp = '\x00' * 256
-
-        if explicit_ramp is not None:
-
-            for i in explicit_ramp:
-                ramp += chr(i)
-
-        else:
-
-            if ramptype == 'cube':
-                # make sure ramplen is big enough, to avoid div-by-0 errors
-                # Not much point in nonlinear if the size is that small, anyway
-                if ramplen >= 8:
-                    table = []
-                    for i in range(ramplen):
-                        table.append(i * i * i)
-                    scale = max(table) / 255.0
-                    for i in range(ramplen):
-                        #print i, table[i], table[i] / scale
-                        ramp += chr(int(table[i] / scale))
-                else:
-                    ramptype = 'linear'
-
-            elif ramptype == 'dcube':
-                if ramplen >= 8:
-                    table = []
-                    for i in range(ramplen / 2 - ramplen, ramplen / 2):
-                        table.append(i * i * i)
-                    adj = abs(min(table))
-                    for i in range(len(table)):
-                        table[i] += adj
-                    scale = max(table) / 255.0
-                    #print "scale:", scale
-                    for i in range(ramplen):
-                        #print i, table[i], table[i] / scale
-                        ramp += chr(int(table[i] / scale))
-                else:
-                    ramptype = 'linear'
-
-            elif ramptype == 'mcube':
-                if ramplen >= 8:
-                    ramplen = (ramplen / 2) * 2 # make sure it's even
-                    table = []
-                    for i in range(ramplen / 2):
-                        table.append(i * i * i)
-                    adj = table[-1]
-                    tmptable = []
-                    for i in table:
-                        tmptable.append(i)
-                    for i in range(1, len(table) + 1):
-                        tmptable.append(abs(table[len(table) - i] - adj) + adj)
-                    table = tmptable
-                    scale = max(table) / 255.0
-                    #print "scale:", scale
-                    for i in range(ramplen):
-                        #print i, table[i], table[i] / scale
-                        ramp += chr(int(table[i] / scale))
-                else:
-                    ramptype = 'linear'
-
-            if ramptype == 'linear':
-                for i in range(ramplen):
-                    ramp += chr(255 * i / ramplen)
-
-        ramp += '\xff' * 256
-
-        old = 0
-        for i in ramp:
-            i = ord(i)
-            if i < old:
-                self.can_fast = False
-                break
-
-            old = i
-        else:
-            self.can_fast = True
-
-        if reverse:
-            ramp = list(ramp)
-            ramp.reverse()
-            ramp = ''.join(ramp)
-
-        return ramp
-        
-        
-    def __init__(self, image, time, ramplen=8, ramptype='linear', ramp=None, reverse=False, alpha=False,
-                 old_widget=None, new_widget=None, **properties):
-
-        super(ImageDissolve, self).__init__(time, **properties)
-
-        self.time = time
-        self.old_widget = old_widget
-        self.new_widget = new_widget
-        self.events = False
-        self.alpha = alpha
-        
-        self.image = renpy.display.im.image(image)
-
-        if ramp is not None:
-            ramplen = len(ramp)
-
-        self.ramp = self.generate_ramp(ramplen, ramptype, ramp, reverse)
-        
-        self.steps = ramplen + 256
-        self.reverse = reverse
-
-
-    def visit(self):
-        return super(ImageDissolve, self).visit() + [ self.image ]
-
-    def render(self, width, height, st, at):
-
-        if renpy.game.less_updates:
-            return null_render(self, width, height, st, at)
-
-        if st >= self.time or not renpy.display.module.can_imageblend:
-            self.events = True
-            return render(self.new_widget, width, height, st, at)
-
-        image = renpy.display.im.cache.get(self.image)
-
-        if st < self.time:
-            renpy.display.render.redraw(self, 0)
-
-        step = int(self.steps * st / self.time)
-
-        if self.reverse:
-            step = self.steps - step
-
-        ramp = self.ramp[step:step+256]
-
-
-        bottom = render(self.old_widget, width, height, st, at)
-        top = render(self.new_widget, width, height, st, at)
-        
-        bottom_surface = bottom.pygame_surface(self.alpha)
-        top_surface = top.pygame_surface(self.alpha)
-
-        iw, ih = image.get_size()
-
-        width = min(bottom.width, top.width, iw)
-        height = min(bottom.height, top.height, ih)
-            
-        def draw(dest, x, y):
-
-            dw, dh = dest.get_size()
-
-            w = min(dw, width + x)
-            h = min(dh, height + y)
-
-            renpy.display.module.imageblend(
-                bottom_surface.subsurface((-x, -y, w, h)),
-                top_surface.subsurface((-x, -y, w, h)),
-                dest.subsurface((0, 0, w, h)),
-                image.subsurface((-x, -y, w, h)),
-                ramp)
-
-        if self.alpha:
-
-            rv = renpy.display.render.Render(width, height)
-            surf = pygame.Surface((width, height), bottom_surface.get_flags(), bottom_surface)
-            draw(surf, 0, 0)
-            renpy.display.render.mutated_surface(surf)
-            rv.blit(surf, (0, 0))
-
-        else:
-            rv = renpy.display.render.Render(width, height, draw_func=draw, opaque=True)
-
-        rv.depends_on(top, True)
-        rv.depends_on(bottom)
-        return rv
 
 def ComposeTransition(trans, before=None, after=None, new_widget=None, old_widget=None):
     if before is not None:
