@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -26,11 +26,9 @@ from pickle import loads
 from cStringIO import StringIO
 import sys
 import types
-import codecs
 
 # Files on disk should be checked before archives. Otherwise, among
 # other things, using a new version of bytecode.rpb will break.
-
 archives = [ ]
 
 # The value of renpy.config.archives the last time index_archives was
@@ -161,11 +159,14 @@ class SubFile(object):
         rv1 = self.start[self.offset:self.offset + length]
         length -= len(rv1)
         self.offset += len(rv1)
-        
-        rv2 = self.f.read(length)        
-        self.offset += len(rv2)
 
-        return rv1 + rv2
+        if length:
+            rv2 = self.f.read(length)        
+            self.offset += len(rv2)
+        else:
+            rv2 = ""
+
+        return (rv1 + rv2)
 
     def readline(self, length=None):
 
@@ -234,13 +235,18 @@ class SubFile(object):
     def seek(self, offset, whence=0):
 
         if whence == 0:
-            self.offset = offset
+            offset = offset
         elif whence == 1:
-            self.offset = self.offset + offset
+            offset = self.offset + offset
         elif whence == 2:
-            self.offset = self.length + offset
+            offset = self.length + offset
 
-        offset = self.offset - len(self.start)
+        if offset > self.length:
+            offset = self.length
+
+        self.offset = offset
+            
+        offset = offset - len(self.start)
         if offset < 0:
             offset = 0
             
@@ -264,6 +270,11 @@ def load(name):
     if renpy.config.reject_backslash and "\\" in name:
         raise Exception("Backslash in filename, use '/' instead: %r" % name)
 
+    if renpy.config.file_open_callback:
+        rv = renpy.config.file_open_callback(name)
+        if rv is not None:
+            return rv
+    
     # Look for the file directly.
     if not renpy.config.force_archives:
 
@@ -305,7 +316,7 @@ def load(name):
             
         return rv
 
-    raise Exception("Couldn't find file '%s'." % name)
+    raise IOError("Couldn't find file '%s'." % name)
 
 def loadable(name):
     """
@@ -334,6 +345,9 @@ def transfn(name):
     if renpy.config.reject_backslash and "\\" in name:
         raise Exception("Backslash in filename, use '/' instead: %r" % name)
 
+    if isinstance(name, unicode):
+        name = name.encode("utf-8")
+    
     for d in renpy.config.searchpath:
         fn = os.path.join(renpy.config.basedir, d, name)
 
