@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -25,8 +25,6 @@
 import codecs
 import re
 import os
-import os.path
-import sets
 
 import renpy
 import renpy.ast as ast
@@ -36,14 +34,45 @@ parse_errors = [ ]
 
 class ParseError(Exception):
 
-    def __init__(self, filename, number, msg, line=None, pos=None):
+    def __init__(self, filename, number, msg, line=None, pos=None, first=False):
         message = u"On line %d of %s: %s" % (number, unicode_filename(filename), msg)
 
-        if line is not None:
-            message += "\n" + line.encode('unicode_escape')
+        if line:
+            lines = line.split('\n')
 
-        if pos is not None:
-            message += "\n" + " " * pos + "^"
+            if len(lines) > 1:                
+                open_string = None
+                i = 0
+                
+                while i < len(lines[0]):
+                    c = lines[0][i]
+
+                    if c == "\\":
+                        i += 1
+                    elif c == open_string:
+                        open_string = None
+                    elif open_string:
+                        pass
+                    elif c == '`' or c == '\'' or c == '"':
+                        open_string = c
+
+                    i += 1
+
+                if open_string:
+                    message += "\n(Perhaps you left out a %s at the end of the first line.)" % open_string
+                    
+            for l in lines:
+                message += "\n" + l
+
+                if pos is not None:
+                    if pos <= len(l):
+                        message += "\n" + " " * pos + "^"
+                        pos = None
+                    else:
+                        pos -= len(l)
+
+                if first:
+                    break
 
         self.message = message
 
@@ -52,6 +81,15 @@ class ParseError(Exception):
     def __unicode__(self):
         return self.message
     
+# Something to hold the expected line number.
+class LineNumberHolder(object):
+    """
+    Holds the expected line number.
+    """
+    
+    def __init__(self):
+        self.line = 0
+
 def unicode_filename(fn):
     """
     Converts the supplied filename to unicode.
@@ -84,6 +122,12 @@ def munge_filename(fn):
     rv = os.path.basename(fn)
     rv = os.path.splitext(rv)[0]
     rv = rv.replace(" ", "_")
+
+    def munge_char(m):
+        return hex(ord(m.group(0)))
+
+    rv = re.sub(r'[^a-zA-Z0-9_]', munge_char, rv)
+
     return "_m1_" + rv + "__"
 
 def list_logical_lines(filename):
@@ -227,9 +271,9 @@ def list_logical_lines(filename):
 
             # print repr(data[pos:])
 
-
-    if not line == "":
-        raise ParseError(filename, start_number, "is not terminated with a newline (check quotes and parenthesis).")
+            
+    if not line == "":        
+        raise ParseError(filename, start_number, "is not terminated with a newline. (Check strings and parenthesis.)", line=line, first=True)
 
     return rv
 
@@ -308,7 +352,7 @@ class Lexer(object):
 
     # A list of keywords which should not be parsed as names, because
     # there is a huge chance of confusion.
-    keywords = sets.Set([
+    keywords = set([
         'as',
         'at',
         'behind',
@@ -329,6 +373,7 @@ class Lexer(object):
         'with',
         'while',
         'zorder',
+        'transform',
         ])
         
 
@@ -341,7 +386,16 @@ class Lexer(object):
         self.eob = False
         
         self.line = -1
-        
+
+        # These are set by advance.
+        self.filename = ""
+        self.text = ""
+        self.number = 0
+        self.subblock = [ ]
+        self.pos = 0
+        self.word_cache_pos = -1
+        self.word_cache_newpos = -1
+        self.word_cache = ""
 
     def advance(self):
         """
@@ -363,7 +417,8 @@ class Lexer(object):
 
         self.filename, self.number, self.text, self.subblock = self.block[self.line]
         self.pos = 0
-
+        self.word_cache_pos = -1
+        
         return True
 
     def match_regexp(self, regexp):
@@ -409,14 +464,19 @@ class Lexer(object):
         return self.match_regexp(regexp) 
         
 
-    def keyword(self, regexp):
+    def keyword(self, word):
         """
         Matches a keyword at the current position. A keyword is a word
         that is surrounded by things that aren't words, like
         whitespace. (This prevents a keyword from matching a prefix.)
         """
 
-        return self.match(regexp + r'\b')
+        oldpos = self.pos
+        if self.word() == word:
+            return word
+
+        self.pos = oldpos
+        return ''
 
 
     def error(self, msg):
@@ -534,20 +594,21 @@ class Lexer(object):
 
         return self.match(r'(\+|\-)?(\d+\.?\d*|\.\d+)([eE][-+]?\d+)?')
 
-    def integer(self):
-        """
-        Tries to parse an integer. Returns a string containing the
-        integer, or None.
-        """
-
-        return self.match(r'(\+|\-)?\d+')
-
     def word(self):
         """
         Parses a name, which may be a keyword or not.
         """
+
+        if self.pos == self.word_cache_pos:
+            self.pos = self.word_cache_newpos
+            return self.word_cache
+
+        self.word_cache_pos = self.pos 
+        rv = self.match(ur'[a-zA-Z_\u00a0-\ufffd][0-9a-zA-Z_\u00a0-\ufffd]*')
+        self.word_cache = rv
+        self.word_cache_newpos = self.pos
         
-        return self.match(ur'[a-zA-Z_\u00a0-\ufffd][0-9a-zA-Z_\u00a0-\ufffd]*')
+        return rv
         
     
     def name(self):
@@ -556,8 +617,6 @@ class Lexer(object):
         """        
 
         oldpos = self.pos
-
-        # rv = self.match(ur'[a-zA-Z_\u00a0-\ufffd][0-9a-zA-Z_\u00a0-\ufffd]*')
         rv = self.word()
         
         if rv in self.keywords:
@@ -675,12 +734,12 @@ class Lexer(object):
         extending to a colon.
         """
 
-        rv = self.delimited_python(':')
+        pe = self.delimited_python(':')
 
-        if not rv:
+        if not pe:
             self.error("expected python_expression")
 
-        rv = renpy.ast.PyExpr(rv.strip(), rv.filename, rv.linenumber)
+        rv = renpy.ast.PyExpr(pe.strip(), pe.filename, pe.linenumber) # E1101
 
         return rv
         
@@ -773,7 +832,8 @@ class Lexer(object):
         """
 
         self.filename, self.number, self.text, self.subblock, self.pos = state 
-
+        self.word_cache_pos = -1
+        
     def get_location(self):
         """
         Returns a (filename, line number) tuple representing the current
@@ -782,7 +842,7 @@ class Lexer(object):
 
         return self.filename, self.number
 
-    def require(self, thing):
+    def require(self, thing, name=None):
         """
         Tries to parse thing, and reports an error if it cannot be done.
 
@@ -792,10 +852,10 @@ class Lexer(object):
         """
 
         if isinstance(thing, str):
-            name = thing
+            name = name or thing
             rv = self.match(thing)            
         else:
-            name = thing.im_func.func_name
+            name = name or thing.im_func.func_name
             rv = thing()
 
         if rv is None:
@@ -825,10 +885,7 @@ class Lexer(object):
 
         rv = [ ]
 
-        # Something to hold the expected line number.
-        class Object(object):
-            pass
-        o = Object()
+        o = LineNumberHolder()
         o.line = self.number
 
         def process(block, indent):
@@ -964,6 +1021,8 @@ def parse_image_specifier(l):
     if layer is None:
         layer = 'master'
 
+    
+        
     return image_name, expression, tag, at_list, layer, zorder, behind
 
 def parse_with(l, node):
@@ -985,7 +1044,7 @@ def parse_with(l, node):
              node,
              ast.With(loc, expr) ]
 
-    
+
     
 def parse_menu(stmtl, loc):
 
@@ -1111,7 +1170,7 @@ def parse_parameters(l):
 
     add_positional = True
 
-    names = sets.Set()
+    names = set()
     
     if not l.match(r'\('):
         return None
@@ -1167,7 +1226,7 @@ def parse_parameters(l):
             parameters.append((name, default))
 
             if add_positional:
-               positional.append(name) 
+                positional.append(name) 
 
         if l.match(r'\)'):
             break
@@ -1366,10 +1425,15 @@ def parse_statement(l):
         if l.keyword('expression'):
             expression = True
             target = l.require(l.simple_expression)
+
         else:
             expression = False            
             target = l.require(l.name)
 
+        # Optional pass, to let someone write:
+        # call expression foo pass (bar, baz)
+        l.keyword('pass')
+            
         arguments = parse_arguments(l)
             
         rv = [ ast.Call(loc, target, expression, arguments) ]
@@ -1387,7 +1451,6 @@ def parse_statement(l):
 
     ### Scene statement.
     if l.keyword('scene'):
-        l.expect_noblock('scene statement')
 
         if l.keyword('onlayer'):
             layer = l.require(l.name)
@@ -1400,7 +1463,13 @@ def parse_statement(l):
             return ast.Scene(loc, None, layer)
 
         imspec = parse_image_specifier(l)
-        rv = parse_with(l, ast.Scene(loc, imspec, imspec[4]))
+        stmt = ast.Scene(loc, imspec, imspec[4])
+        rv = parse_with(l, stmt)
+
+        if l.match(':'):
+            stmt.atl = renpy.atl.parse_atl(l.subblock_lexer())            
+        else:
+            l.expect_noblock('scene statement')
 
         l.expect_eol()
         l.advance()
@@ -1410,10 +1479,15 @@ def parse_statement(l):
     ### Show statement.
     if l.keyword('show'):
         imspec = parse_image_specifier(l)
-        rv = parse_with(l, ast.Show(loc, imspec))
+        stmt = ast.Show(loc, imspec)
+        rv = parse_with(l, stmt)
 
+        if l.match(':'):
+            stmt.atl = renpy.atl.parse_atl(l.subblock_lexer())            
+        else:
+            l.expect_noblock('show statement')
+        
         l.expect_eol()
-        l.expect_noblock('show statement')
         l.advance()
 
         return rv
@@ -1441,18 +1515,80 @@ def parse_statement(l):
     ### Image statement.
     if l.keyword('image'):
 
-        if not l.init:
-            l.error('image statements may only appear in init blocks.')
-        
         name = parse_image_name(l)
+
+        if l.match(':'):
+            l.expect_eol()
+            expr = None
+            atl = renpy.atl.parse_atl(l.subblock_lexer())
+        else:            
+            l.require('=')
+            expr = l.rest()
+            atl = None
+            l.expect_noblock('image statement')
+
+        rv = ast.Image(loc, name, expr, atl)
+
+        if not l.init:
+            rv = ast.Init(loc, [ rv ], 990)        
+        
+        l.advance()
+
+        return rv
+
+    ### Define statement.
+    if l.keyword('define'):
+
+        priority = l.integer()
+        if priority:
+            priority = int(priority)
+        else:
+            priority = 0
+        
+        name = l.require(l.name)
         l.require('=')
         expr = l.rest()
 
-        l.expect_noblock('image statement')
+        l.expect_noblock('define statement')
+
+        rv = ast.Define(loc, name, expr)
+
+        if not l.init:
+            rv = ast.Init(loc, [ rv ], priority)        
+        
         l.advance()
 
-        return ast.Image(loc, name, expr)
+        return rv
 
+    ### Transform statement.
+    if l.keyword('transform'):
+        
+        priority = l.integer()
+        if priority:
+            priority = int(priority)
+        else:
+            priority = 0
+        
+        name = l.require(l.name)
+        parameters = parse_parameters(l)
+
+        if parameters and (parameters.extrakw or parameters.extrapos):
+            l.error('transform statement does not take a variable number of parameters')
+
+        l.require(':')
+        l.expect_eol()
+
+        atl = renpy.atl.parse_atl(l.subblock_lexer())
+
+        rv = ast.Transform(loc, name, atl, parameters)
+
+        if not l.init:
+            rv = ast.Init(loc, [ rv ], priority)        
+        
+        l.advance()
+
+        return rv
+    
     ### One-line python statement.
     if l.match(r'\$'):
         python_code = l.rest()
@@ -1559,6 +1695,20 @@ def parse_statement(l):
         
     l.revert(state)
 
+    # Try parsing as the default statement.
+    if () in renpy.statements.registry:
+        text = l.text
+        l.expect_noblock('default statement')
+        l.advance()
+
+        renpy.exports.push_error_handler(l.error)
+        try:
+            rv = ast.UserStatement(loc, text)
+        finally:
+            renpy.exports.pop_error_handler()
+
+        return rv
+    
     # The one and two arguement say statements are ambiguous in terms
     # of lookahead. So we first try parsing as a one-argument, then a
     # two-argument.
@@ -1631,9 +1781,13 @@ def parse(fn):
 
     renpy.game.exception_info = 'While parsing ' + fn + '.'
 
-    lines = list_logical_lines(fn)
-    nested = group_logical_lines(lines)
-
+    try:
+        lines = list_logical_lines(fn)
+        nested = group_logical_lines(lines)
+    except ParseError, e:
+        parse_errors.append(e.message)
+        return None
+        
     l = Lexer(nested)
 
     rv = parse_block(l)
@@ -1676,7 +1830,7 @@ def report_parse_errors():
         if renpy.config.editor:
             renpy.exports.launch_editor([ 'errors.txt' ], 1, transient=1)
         else:
-            os.startfile('errors.txt')
+            os.startfile('errors.txt') # E1101
     except:
         pass
         
