@@ -1,4 +1,4 @@
-# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -31,11 +31,9 @@ import compiler.ast as ast
 
 import marshal
 import random
-import types
 import weakref
 import re
 import sets
-import sys
 
 import renpy
 
@@ -74,9 +72,9 @@ def reached(obj, reachable, wait):
         pass
 
     try:
-       # Treat as iterable
+        # Treat as iterable
         if not isinstance(obj, basestring):
-            for v in obj:
+            for v in obj.__iter__():
                 reached(v, reachable, wait)
     except:
         pass
@@ -220,13 +218,19 @@ def py_compile(source, mode, filename='<none>', lineno=1):
 
     if "\\u" in source:
         source = string_re.sub(make_unicode, source)
-    
+        
+    if mode == "exec":
+        source = "from __future__ import with_statement\n" + source
+        line_offset = 2
+    else:
+        line_offset = 1
+        
     try:
         tree = parse(source, mode)
     except SyntaxError, e:
         
         if e.lineno is not None:
-            msg = "Syntax error on line %d of %s" % (e.lineno + lineno - 1, filename)
+            msg = "Syntax error on line %d of %s" % (e.lineno + lineno - line_offset, filename)
 
             if len(source) < 128:
                 msg += ":\n    " + orig_source
@@ -238,10 +242,10 @@ def py_compile(source, mode, filename='<none>', lineno=1):
     recursively_replace(tree, wrap_node)
 
     if mode == 'exec':
-        set_filename(filename, lineno - 1, tree)
+        set_filename(filename, lineno - line_offset, tree)
         cg = ModuleCodeGenerator(tree)
     else:
-        set_filename(filename, lineno - 1, tree)
+        set_filename(filename, lineno - line_offset, tree)
         cg = ExpressionCodeGenerator(tree)
 
     return cg.getCode()
@@ -295,9 +299,9 @@ class RevertableList(list):
     reverse = mutator(list.reverse)
     sort = mutator(list.sort)
 
-    def wrapper(method):
-        def newmethod(*args, **kwargs):
-            return RevertableList(method(*args, **kwargs))
+    def wrapper(method): # E0213
+        def newmethod(*args, **kwargs): 
+            return RevertableList(method(*args, **kwargs)) # E1102
 
         return newmethod
 
@@ -305,6 +309,8 @@ class RevertableList(list):
     __getslice__ = wrapper(list.__getslice__)
     __mul__ = wrapper(list.__mul__)
 
+    del wrapper
+    
     def get_rollback(self):
         return self[:]
 
@@ -326,9 +332,9 @@ class RevertableDict(dict):
     popitem = mutator(dict.popitem)
     setdefault = mutator(dict.setdefault)
 
-    def list_wrapper(method):
+    def list_wrapper(method): # E0213
         def newmethod(*args, **kwargs):
-            return RevertableList(method(*args, **kwargs))
+            return RevertableList(method(*args, **kwargs)) # E1102
 
         return newmethod
 
@@ -336,6 +342,8 @@ class RevertableDict(dict):
     values = list_wrapper(dict.values)
     items = list_wrapper(dict.items)
     
+    del list_wrapper
+
     def copy(self):
         rv = RevertableDict()
         rv.update(self)
@@ -367,11 +375,9 @@ class RevertableSet(sets.Set):
     union_update = mutator(sets.Set.union_update)
     update = mutator(sets.Set.update)
 
-    # TODO: Wrap all of the methods that can return normal sets.
-
-    def wrapper(method):
+    def wrapper(method): # E0213
         def newmethod(*args, **kwargs):
-            rv = method(*args, **kwargs)
+            rv = method(*args, **kwargs) # E1102
             if isinstance(rv, sets.Set):
                 return RevertableSet(rv)
             else:
@@ -404,7 +410,6 @@ class RevertableSet(sets.Set):
 class RevertableObject(object):
 
     def __setattr__(self, attr, value):
-        import sys
         object.__setattr__(self, attr, value)
 
     def __delattr__(self, attr):
@@ -491,6 +496,9 @@ class Rollback(renpy.object.Object):
     """
 
     def __init__(self):
+
+        super(Rollback, self).__init__()
+
         self.context = renpy.game.contexts[0].rollback_copy()
         self.objects = [ ]
         self.store = [ ]
@@ -539,7 +547,8 @@ class Rollback(renpy.object.Object):
             else:
                 if renpy.config.debug:
                     print "Removing unreachable:", o
-                pass
+
+                    pass
                 
         self.objects = new_objects
 
@@ -595,16 +604,19 @@ class RollbackLog(renpy.object.Object):
     nosave = [ 'old_store', 'mutated' ]
 
     def __init__(self):
+
+        super(RollbackLog, self).__init__()
+
         self.log = [ ]
         self.current = None
         self.mutated = { }
         self.ever_been_changed = { }
         self.rollback_limit = 0
         self.forward = [ ]
-
+        self.old_store = { }
+        
         # Did we just do a roll forward?
         self.rolled_forward = False
-
         
         # Reset the RNG on the creation of a new game.
         rng.reset()
@@ -752,10 +764,14 @@ class RollbackLog(renpy.object.Object):
         that the user may want to rollback to just before this
         node.
         """
-
+        
         if self.current.checkpoint:
             return
-            
+
+        # Only allow checkpoints in the top-level context.
+        if len(renpy.game.contexts) > 1:
+            return
+        
         if self.rollback_limit < renpy.config.hard_rollback_limit: 
             self.rollback_limit += 1
 
@@ -950,11 +966,16 @@ def py_eval(source, globals=None, locals=None):
     return eval(py_compile(source, 'eval'), globals, locals)
 
 
+
 # Code for pickling bound methods.
 
 def method_pickle(method):
     name = method.im_func.__name__
-    obj = method.im_class
+
+    obj = method.im_self
+
+    if obj is None:
+        obj = method.im_class
 
     return method_unpickle, (obj, name)
 
