@@ -1,4 +1,4 @@
-# Copyright 2004-2009 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -36,8 +36,6 @@
 # Empty lines are ignored. (This lets us clear out outstanding errors.)
 
 import renpy
-import sys
-import select
 import os
 import os.path
 
