@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -31,6 +31,16 @@ prefixes = [ 'hover_', 'idle_', 'insensitive_', 'activate_', ]
 # A map from prefix to priority and alternates.
 prefix_subs = { }
 
+# These allow quick access to important properties.
+prop_xpos = None
+prop_ypos = None
+prop_xanchor = None
+prop_yanchor = None
+prop_xoffset = None
+prop_yoffset = None
+prop_subpixel = None
+
+
 def register_prefix(prefix, prio, addprefixes=[]):
 
     for r in roles:
@@ -72,6 +82,34 @@ def none_is_null(d):
     else:
         return renpy.easy.displayable(d)
 
+# This expands the outlines list.
+def expand_outlines(l):
+    rv = [ ]
+
+    for i in l:
+        if len(i) == 2:
+            rv.append((i[0], renpy.easy.color(i[1]), 0, 0))
+        else:
+            rv.append((i[0], renpy.easy.color(i[1]), i[2], i[3]))
+
+    return rv
+
+# Names for anchors.
+anchors = dict(
+    left=0.0,
+    right=1.0,
+    center=0.5,
+    top=0.0,
+    bottom=1.0,
+    )
+
+def expand_anchor(v):
+    """
+    Turns an anchor into a number.
+    """
+
+    return anchors.get(v, v)
+    
 # A map of properties that we know about. The properties may take a
 # function that is called to convert the argument to something more
 # useful.
@@ -82,6 +120,7 @@ style_properties = dict(
     background = renpy.easy.displayable_or_none,
     bar_invert = None,
     bar_resizing = None,
+    unscrollable = None,
     bar_vertical = None,
     black_color = renpy.easy.color,
     bold = None,
@@ -103,6 +142,7 @@ style_properties = dict(
     foreground = renpy.easy.displayable_or_none,
     sound = None,
     italic = None,
+    justify = None,
     language = None,
     layout = None,
     left_margin = None,
@@ -110,6 +150,7 @@ style_properties = dict(
     line_spacing = None,
     mouse = None,
     min_width = None,
+    outlines = expand_outlines,
     rest_indent = None,
     right_margin = None,
     right_padding = None,
@@ -121,6 +162,7 @@ style_properties = dict(
     spacing = None,
     strikethrough = None,
     subtitle_width = None,
+    subpixel = None,
     text_y_fudge = None,
     text_align = None,
     thumb = none_is_null,
@@ -130,13 +172,13 @@ style_properties = dict(
     top_margin = None,
     top_padding = None,
     underline = None,
-    xanchor = None,
+    xanchor = expand_anchor,
     xfill = None,
     xmaximum = None,
     xminimum = None,
     xoffset = None,
     xpos = None,
-    yanchor = None,
+    yanchor = expand_anchor,
     yfill = None,
     ymaximum = None,
     yminimum = None,
@@ -211,7 +253,7 @@ def init():
             for p in prefixes + [ '' ]:
                 expansions[r + p + k] = [ a for b in substitutes[k] for a in expansions[r + p + b] ]
 
-    for i in ('xpos', 'xanchor', 'xoffset', 'ypos', 'yanchor', 'yoffset'):
+    for i in ('xpos', 'xanchor', 'xoffset', 'ypos', 'yanchor', 'yoffset', 'subpixel'):
         globals()["prop_" + i] = property_number[i]
                 
             
@@ -292,6 +334,9 @@ class StyleManager(object):
 
         s = Style(parent, { }, heavy=True, help=description)
         setattr(self, name, s)
+
+    def rebuild(self):
+        renpy.style.rebuild()
         
     def exists(self, name):
         """
@@ -342,7 +387,6 @@ def expand_properties(properties):
     return rv
 
 
-
 # This builds the style. 
 def build_style(style):
     
@@ -438,10 +482,6 @@ def build_style(style):
                 build_style(ss)
 
             updates.extend(ss.updates)
-            
-
-        
-            
         
     style.updates = my_updates = [ ]
         
@@ -471,12 +511,18 @@ def build_styles():
     styles_pending = None
     styles_built = True
 
-def rebuild():
 
-    global style_pending
+def rebuild():
+    global styles_pending
     global styles_built
 
-    styles_pending = [ j for i in style_parts.values() for j in i.values() ]
+    if renpy.game.init_phase:
+        return
+
+    if styles_pending is None:
+        styles_pending = [ ]
+    
+    styles_pending += [ j for i in style_parts.values() for j in i.values() ]
     styles_built = False
     
     for i in styles_pending:
@@ -484,6 +530,7 @@ def rebuild():
         
     build_styles()
 
+    
 def backup():
     rv = { }
     
@@ -492,7 +539,8 @@ def backup():
             rv[first, rest] = (v.parent, v.properties[:])
 
     return rv
-        
+
+
 def restore(o):
     global styles_built
     global styles_pending
@@ -505,28 +553,29 @@ def restore(o):
         style_parts[first][rest].properties = properties[:]
         styles_pending.append(style_parts[first][rest])
 
+
 def style_metaclass(name, bases, attrs):
 
     for k in expansions.iterkeys():
-        def setter(self, v,  k=k):
+        def setter_a(self, v,  k=k):
             self.setattr(k, v)
 
-        def deleter(self, k=k):
+        def deleter_a(self, k=k):
             self.delattr(k)
 
-        attrs[k] = property(None, setter, deleter)
+        attrs[k] = property(None, setter_a, deleter_a)
     
     for k, number in property_number.iteritems():
-        def getter(self, number=number):
+        def getter_b(self, number=number):
             return self.cache[self.offset + number]
 
-        def setter(self, v,  k=k):
+        def setter_b(self, v,  k=k):
             self.setattr(k, v)
 
-        def deleter(self, k=k):
+        def deleter_b(self, k=k):
             self.delattr(k)
 
-        attrs[k] = property(getter, setter, deleter)
+        attrs[k] = property(getter_b, setter_b, deleter_b)
 
     return type(name, bases, attrs)
 
@@ -580,7 +629,7 @@ class Style(object):
             if isinstance(parent, basestring):
                 parent = ( parent, )
             if isinstance(parent, Style):
-                parent = parent.name
+                parent = parent.name # E1103
 
                 if parent is None:
                     raise Exception("The parent of a style must be a named style.")
@@ -598,6 +647,7 @@ class Style(object):
         else:
             self.name = ( name, )
 
+        self.parent = None
         self.set_parent(parent)
             
         self.indexed = None
@@ -686,6 +736,7 @@ class Style(object):
             c[o + prop_yanchor],
             c[o + prop_xoffset],
             c[o + prop_yoffset],
+            c[o + prop_subpixel],
             )
     
 
@@ -704,9 +755,6 @@ def write_text(filename):
     styles = style_map.items()
     styles.sort()
 
-
-    style_props = { }
-
     for name, sty in styles:
 
         if not isinstance(sty, Style):
