@@ -1,4 +1,4 @@
-# Copyright 2004-2008 PyTom <pytom@bishoujo.us>
+# Copyright 2004-2010 PyTom <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -26,12 +26,11 @@
 # but are probably better than nothing.
 
 import sys
-import os
-import encodings.zlib_codec
+import encodings.zlib_codec; encodings.zlib_codec # E0601
 import random
 import glob
 
-from cPickle import loads, dumps, HIGHEST_PROTOCOL
+from cPickle import dumps, HIGHEST_PROTOCOL
 
 # The amount of padding we will add.
 padding_max = 64
@@ -61,7 +60,7 @@ def archive(prefix, files):
 
     key = random.randint(0, 0x7ffffffe)
     
-    padding = "RPA-3.0 XXXXXXXXXXXXXXXX XXXXXXXX\n"
+    padding = "RPA-3.0 XXXXXXXXXXXXXXXX XXXXXXXX\n" # W0511
 
     archivef.write(padding)
     offset = len(padding)
@@ -71,8 +70,8 @@ def archive(prefix, files):
 
     for fullfn, shortfn in files:
 
-        if shortfn.lower().endswith(".ttf"):
-            continue
+        # if shortfn.lower().endswith(".ttf"):
+        #    continue
         
         shortfn = shortfn.replace("\\", "/")
 
